"""
Sidebar Component Python Integration
"""

from typing import Dict, Any, Optional, List
from ...base import Component
from .....core.theme import get_theme, get_color
from .....core.state import State, get_state
from ...navigation.nav_item.nav_item import NavItem

import json

# Debug flag for Sidebar component
SIDEBAR_DEBUG = False

# Use named global states for proper state synchronization
current_page_state = get_state("current_page", "home")
sidebar_expanded_state = get_state("sidebar_expanded", True)

class Sidebar(Component):
    def __init__(self, nav_items: List[NavItem], app_title: str = "Cacao App", 
                 show_footer: bool = True, footer_text: str = "© 2025 Cacao Framework") -> None:
        """Initialize sidebar with navigation items.
        
        Args:
            nav_items: List of NavItem components
            app_title: Title to display in the sidebar header
            show_footer: Whether to show the footer in the sidebar
            footer_text: Custom text to display in the footer (if shown)
        """
        # The theme will be set by the parent Sidebar component
        super().__init__()
        self.nav_items = nav_items
        self.app_title = app_title
        self.show_footer = show_footer
        self.footer_text = footer_text
        
    def render(self) -> Dict[str, Any]:
        # Get theme colors
        theme = get_theme()
        theme_colors = theme.get('colors', {})
        
        children = [
            # App header/brand section
            {
                "type": "div",
                "props": {
                    "style": {
                        "padding": "20px 16px",
                        "borderBottom": f"1px solid {self.parent.styles.get('sidebar_border', get_color('sidebar_border'))}",
                        "display": "flex",
                        "alignItems": "center",
                        "height": "64px",
                        "backgroundColor": self.parent.styles.get("sidebar_header_bg", get_color("sidebar_header_bg"))
                    },
                    "children": [
                        {
                            "type": "h2",
                            "props": {
                                "content": self.app_title,
                                "style": {
                                    "margin": 0,
                                    "fontSize": "18px",
                                    "fontWeight": "600",
                                    "color": self.parent.styles.get("app_title_color", theme_colors.get("app_title_color", "#D6C3B6"))
                                }
                            }
                        }
                    ]
                }
            },
            # Navigation items container
            {
                "type": "div",
                "props": {
                    "style": {
                        "padding": "16px 0",
                        "flex": 1,
                        "overflowY": "auto"
                    },
                    "children": [nav_item.render() for nav_item in self.nav_items]
                }
            }
        ]
        
        # Add footer if enabled
        if self.show_footer:
            children.append({
                "type": "div",
                "props": {
                    "style": {
                        "borderTop": f"1px solid {self.parent.styles.get('sidebar_border', get_color('sidebar_border'))}",
                        "padding": "16px",
                        "fontSize": "12px",
                        "color": self.parent.styles.get("sidebar_text", get_color("sidebar_text"))
                    },
                    "children": [
                        {
                            "type": "text",
                            "props": {
                                "content": self.footer_text,
                                "style": {
                                    "margin": 0
                                }
                            }
                        }
                    ]
                }
            })
        
        return {
            "type": "sidebar",
            "key": "sidebar",
            "props": {
                "style": {
                    "width": self.parent.styles.get("sidebar_width", "250px") if sidebar_expanded_state.value else self.parent.styles.get("sidebar_collapsed_width", "64px"),
                    "height": "100vh",
                    "position": "fixed",
                    "top": 0,
                    "left": 0,
                    "backgroundColor": self.parent.styles.get("sidebar_bg", get_color("sidebar_bg")),
                    "color": self.parent.styles.get("active_text", get_color("active_text")),
                    "boxShadow": "0 0 15px rgba(107, 66, 38, 0.15)",
                    "transition": "width 0.3s ease",
                    "padding": "0",
                    "display": "flex",
                    "flexDirection": "column",
                    "zIndex": 1000
                },
                "children": children
            }
        }


"""
Nav Item Component Python Integration
"""

from typing import Dict, Any, Optional
import json
from ...base import Component
from .....core.theme import get_theme, get_color

# Debug flag for NavItem component
NAV_ITEM_DEBUG = False

class NavItem(Component):
    def __init__(self, id: str, label: str, icon: Optional[str] = None, is_active: bool = False) -> None:
        super().__init__()
        self.id = id
        self.label = label
        self.icon = icon
        self.is_active = is_active

    def render(self) -> Dict[str, Any]:
        # Get styles from parent component or use global theme
        parent_styles = {}
        if hasattr(self, 'parent') and hasattr(self.parent, 'styles'):
            parent_styles = self.parent.styles
        
        # Get global theme and colors
        theme = get_theme()
        theme_colors = theme.get('colors', {})
        
        # Base and active styles
        base_style = {
            "display": "flex",
            "alignItems": "center",
            "padding": parent_styles.get("nav_item_padding", "12px 16px"),
            "margin": "4px 8px",
            "borderRadius": "8px",
            "cursor": "pointer",
            "transition": "all 0.2s ease",
            "color": parent_styles.get("sidebar_text", theme_colors.get("sidebar_text", "#D6C3B6")),
            "fontSize": parent_styles.get("nav_item_size", "15px"),
            "fontWeight": parent_styles.get("nav_item_weight", "500"),
            "textDecoration": "none",
        }
        
        # Apply active styles when item is selected
        if self.is_active:
            # Get theme colors with proper fallbacks
            theme = get_theme()
            theme_colors = theme.get('colors', {})
            active_styles = {
                "backgroundColor": parent_styles.get("active_bg", theme_colors.get("active_bg", "#D6C3B6")),
                "color": parent_styles.get("active_text", theme_colors.get("active_text", "#FFFFFF")),
                "boxShadow": "0 2px 5px rgba(107, 66, 38, 0.3)"
            }
            # Merge active styles into base styles
            base_style.update(active_styles)
        else:
            # Hover effect will be handled by CSS in the real app
            # Here we're defining the non-active state
            base_style["backgroundColor"] = "transparent"

        if NAV_ITEM_DEBUG:
            # --- Debugging Start ---
            print(f"[DEBUG NavItem {self.id}] is_active: {self.is_active}")

        # Ensure hover style doesn't interfere if present
        final_style = {k: v for k, v in base_style.items() if k != "&:hover"}

        if NAV_ITEM_DEBUG:
            print(f"[DEBUG NavItem {self.id}] Final style prop: {json.dumps(final_style, indent=2)}")
            # --- Debugging End ---
                
        # Create the icon element if provided
        icon_element = None
        if self.icon:
            icon_element = {
                "type": "div",
                "props": {
                    "style": {
                        "width": "28px",
                        "height": "28px", 
                        "display": "flex",
                        "alignItems": "center",
                        "justifyContent": "center",
                        "marginRight": "14px",
                        "backgroundColor": parent_styles.get("active_icon_bg", theme_colors.get("active_icon_bg", "#8B5E41")) if self.is_active else parent_styles.get("inactive_icon_bg", theme_colors.get("inactive_icon_bg", "rgba(107, 66, 38, 0.3)")),
                        "color": parent_styles.get("active_text", theme_colors.get("active_text", "#FFFFFF")),
                        "borderRadius": "6px",
                        "fontSize": "16px",
                        "fontWeight": "bold"
                    },
                    "children": [{
                        "type": "text",
                        "props": {
                            "content": self.icon,
                            "style": {
                                "color": parent_styles.get("active_text", get_color("active_text"))
                            }
                        }
                    }]
                }
            }

        children = []
        if icon_element:
            children.append(icon_element)
            
        # Add the label with improved visibility
        children.append({
            "type": "text",
            "props": {
                "content": self.label,
                "style": {
                    "whiteSpace": "nowrap",
                    "overflow": "hidden",
                    "textOverflow": "ellipsis",
                    "fontWeight": "500",
                    "fontSize": "15px",
                    "color": parent_styles.get("active_text", get_color("active_text")) if self.is_active else parent_styles.get("sidebar_text", get_color("sidebar_text"))
                }
            }
        })
        
        return {
            "type": "nav_item",
            "key": f"nav-{self.id}",
            "props": {
                "style": final_style, # Use the debugged style dict
                "children": children,
                "onClick": {
                    "action": "set_state",
                    "state": "current_page",
                    "value": self.id,
                    "immediate": True  # Signal that this state change should be applied immediately
                }
            }
        }


from typing import Any, Dict, Optional
from ...base import Component

class TreeViewer(Component):
    """
    A component to render collapsible tree structures.

    Props:
      - id: unique component ID
      - data: the object to display as a tree
      - expand_all: whether to start fully expanded
      - theme: "light" or "dark"
      - on_node_click: optional event name to fire when a key is clicked
    """
    def __init__(
        self,
        id: str,
        data: Dict[str, Any],
        expand_all: bool = False,
        theme: str = "light",
        on_node_click: Optional[str] = None,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.id = id
        self.data = data
        self.expand_all = expand_all
        self.theme = theme
        self.on_node_click = on_node_click

    def validate(self) -> bool:
        return isinstance(self.data, (dict, list))

    def to_dict(self) -> Dict[str, Any]:
        props = {
            "id": self.id,
            "data": self.data,
            "expand_all": self.expand_all,
            "theme": self.theme,
        }
        if self.on_node_click:
            props["on_node_click"] = self.on_node_click

        return {
            "type": "tree_viewer",
            "props": props,
            **self.get_base_props()
        }



"""
Collapse component for hiding and showing content sections.
"""

from typing import Any, Dict
from ...base import Component


class Collapse(Component):
    """
    Hides and shows content sections.
    """
    def __init__(
        self,
        panels,
        accordion = False,
        active_key = None,
        **kwargs
    ) -> None:
        self.panels = panels
        self.accordion = accordion
        self.active_key = active_key if active_key is not None else []
        self.extra_props = kwargs

    def render(self) -> Dict[str, Any]:
        return {
            "type": "collapse",
            "props": {
                "panels": self.panels,
                "accordion": self.accordion,
                "activeKey": self.active_key,
                **self.extra_props
            }
        }


"""
React Component Example for Cacao.
Demonstrates how to use React components from npm packages in Cacao applications.
This example shows how to integrate CodeMirror, a popular code editor.
"""

import cacao
from cacao.ui import ReactComponent
from cacao.extensions.react_extension import ReactExtension

# Create the app with React extension
app = cacao.App(extensions=[ReactExtension()])

@app.mix("/")
def home():
    """Render the home page with a CodeMirror editor."""
    return {
        "type": "div",
        "props": {
            "style": {
                "padding": "20px",
                "fontFamily": "Arial, sans-serif",
                "maxWidth": "800px",
                "margin": "0 auto"
            }
        },
        "children": [
            {
                "type": "h1",
                "props": {
                    "content": "React Component Integration Example",
                    "style": {
                        "color": "#6B4226",
                        "marginBottom": "20px"
                    }
                }
            },
            {
                "type": "p",
                "props": {
                    "content": "This example demonstrates how to use React components from npm packages in Cacao applications.",
                    "style": {
                        "marginBottom": "20px"
                    }
                }
            },
            # CodeMirror React component
            # Reverting to original CodeMirror v5
            ReactComponent(
                package="codemirror",
                component="CodeMirror", # The component name is the global object itself for v5
                props={
                    "value": """// CodeMirror Example
function helloWorld() {
    console.log("Hello from CodeMirror in Cacao!");
    return "Hello, world!";
}

// Try editing this code!
const result = helloWorld();
""",
                    "options": {
                        "mode": "javascript",
                        "theme": "material",
                        "lineNumbers": True,
                        "lineWrapping": True
                    }
                },
                css=["lib/codemirror.css", "theme/material.css", "mode/javascript/javascript.css"],
                id="code-editor",
                version="5.65.15",  # Using stable v5
                cdn="https://cdn.jsdelivr.net/npm"  # Use default jsdelivr CDN
            ).render(),
            {
                "type": "div",
                "props": {
                    "style": {
                        "marginTop": "20px",
                        "padding": "15px",
                        "backgroundColor": "#f5f5f5",
                        "borderRadius": "4px"
                    }
                },
                "children": [
                    {
                        "type": "h3",
                        "props": {
                            "content": "How It Works"
                        }
                    },
                    {
                        "type": "p",
                        "props": {
                            "content": "The ReactComponent class creates a bridge between Cacao and React, allowing you to use React components from npm packages."
                        }
                    },
                    {
                        "type": "pre",
                        "props": {
                            "content": """# Example usage
editor = ReactComponent(
    package="codemirror",
    component="CodeMirror",
    props={
        "value": "const hello = 'world';",
        "options": {
            "mode": "javascript",
            "theme": "material",
            "lineNumbers": True
        }
    },
    css=["lib/codemirror.css", "theme/material.css"]
)""",
                            "style": {
                                "backgroundColor": "#f0f0f0",
                                "padding": "10px",
                                "borderRadius": "4px",
                                "overflow": "auto"
                            }
                        }
                    }
                ]
            }
        ]
    }

if __name__ == "__main__":
    app.brew()


"""
Breadcrumb Component

A breadcrumb navigation component that shows the current page's location within 
a navigational hierarchy. Supports various separators, icons, and styling options.
"""

from typing import List, Dict, Any, Optional, Union
from cacao.ui.components.base import Component
from cacao.ui.types import ComponentState, ValidationResult


class Breadcrumb(Component):
    """
    A breadcrumb navigation component for showing hierarchical navigation paths.
    
    Features:
    - Customizable breadcrumb items with labels and URLs
    - Multiple separator styles (arrow, slash, dot, custom)
    - Icon support for breadcrumb items
    - Clickable and non-clickable items
    - Responsive design with overflow handling
    - Customizable styling and themes
    - Accessibility support with proper ARIA attributes
    """
    
    def __init__(
        self,
        id: str,
        items: List[Dict[str, Any]] = None,
        separator: str = 'arrow',
        separator_icon: Optional[str] = None,
        show_home: bool = True,
        home_url: str = '/',
        home_icon: str = 'home',
        max_items: Optional[int] = None,
        responsive: bool = True,
        size: str = 'medium',
        variant: str = 'default',
        on_click: Optional[str] = None,
        **kwargs
    ):
        """
        Initialize a Breadcrumb component.
        
        Args:
            id: Unique identifier for the component
            items: List of breadcrumb items with labels and URLs
            separator: Type of separator ('arrow', 'slash', 'dot', 'custom')
            separator_icon: Custom icon for separator (when separator='custom')
            show_home: Whether to show a home icon at the beginning
            home_url: URL for the home link
            home_icon: Icon class for the home link
            max_items: Maximum number of items to show before collapsing
            responsive: Whether to enable responsive behavior
            size: Size of the breadcrumb ('small', 'medium', 'large')
            variant: Style variant ('default', 'outlined', 'ghost')
            on_click: JavaScript function to call when item is clicked
        """
        super().__init__(id, **kwargs)
        
        self.items = items or []
        self.separator = separator
        self.separator_icon = separator_icon
        self.show_home = show_home
        self.home_url = home_url
        self.home_icon = home_icon
        self.max_items = max_items
        self.responsive = responsive
        self.size = size
        self.variant = variant
        self.on_click = on_click
        
        # Validate component
        self._validate_component()
    
    def _validate_component(self) -> ValidationResult:
        """Validate the breadcrumb component configuration."""
        errors = []
        
        # Validate separator
        valid_separators = ['arrow', 'slash', 'dot', 'custom']
        if self.separator not in valid_separators:
            errors.append(f"Invalid separator: {self.separator}. Must be one of: {valid_separators}")
        
        # Validate custom separator icon
        if self.separator == 'custom' and not self.separator_icon:
            errors.append("Custom separator requires separator_icon to be specified")
        
        # Validate size
        valid_sizes = ['small', 'medium', 'large']
        if self.size not in valid_sizes:
            errors.append(f"Invalid size: {self.size}. Must be one of: {valid_sizes}")
        
        # Validate variant
        valid_variants = ['default', 'outlined', 'ghost']
        if self.variant not in valid_variants:
            errors.append(f"Invalid variant: {self.variant}. Must be one of: {valid_variants}")
        
        # Validate items
        if self.items:
            for i, item in enumerate(self.items):
                if not isinstance(item, dict):
                    errors.append(f"Item {i} must be a dictionary")
                    continue
                
                if 'label' not in item:
                    errors.append(f"Item {i} must have a 'label' field")
        
        # Validate max_items
        if self.max_items is not None and self.max_items < 1:
            errors.append("max_items must be at least 1")
        
        return ValidationResult(is_valid=len(errors) == 0, errors=errors)
    
    def add_item(self, label: str, url: str = None, icon: str = None, 
                 active: bool = False, disabled: bool = False) -> 'Breadcrumb':
        """
        Add a new breadcrumb item.
        
        Args:
            label: Display label for the item
            url: URL for the item (if clickable)
            icon: Icon class for the item
            active: Whether this is the current/active item
            disabled: Whether the item is disabled
        
        Returns:
            Self for method chaining
        """
        item = {
            'label': label,
            'active': active,
            'disabled': disabled
        }
        
        if url:
            item['url'] = url
        
        if icon:
            item['icon'] = icon
        
        self.items.append(item)
        return self
    
    def remove_item(self, index: int) -> 'Breadcrumb':
        """
        Remove a breadcrumb item by index.
        
        Args:
            index: Index of the item to remove
        
        Returns:
            Self for method chaining
        """
        if 0 <= index < len(self.items):
            self.items.pop(index)
        
        return self
    
    def set_active_item(self, index: int) -> 'Breadcrumb':
        """
        Set an item as active (current page).
        
        Args:
            index: Index of the item to make active
        
        Returns:
            Self for method chaining
        """
        for i, item in enumerate(self.items):
            item['active'] = (i == index)
        
        return self
    
    def clear_items(self) -> 'Breadcrumb':
        """
        Clear all breadcrumb items.
        
        Returns:
            Self for method chaining
        """
        self.items = []
        return self
    
    def get_active_item(self) -> Optional[Dict[str, Any]]:
        """
        Get the currently active breadcrumb item.
        
        Returns:
            Active item or None if no active item
        """
        for item in self.items:
            if item.get('active'):
                return item
        return None
    
    def get_clickable_items(self) -> List[Dict[str, Any]]:
        """
        Get all clickable breadcrumb items.
        
        Returns:
            List of clickable items
        """
        return [item for item in self.items if item.get('url') and not item.get('disabled')]
    
    def get_visible_items(self) -> List[Dict[str, Any]]:
        """
        Get the items that should be visible based on max_items setting.
        
        Returns:
            List of visible items
        """
        if not self.max_items or len(self.items) <= self.max_items:
            return self.items
        
        # Show first item, ellipsis, and last (max_items - 1) items
        if self.max_items <= 2:
            return self.items[-self.max_items:]
        
        visible_items = []
        
        # Add first item
        if self.items:
            visible_items.append(self.items[0])
        
        # Add ellipsis if needed
        if len(self.items) > self.max_items:
            visible_items.append({'label': '...', 'ellipsis': True})
        
        # Add last (max_items - 2) items
        start_index = len(self.items) - (self.max_items - 2)
        visible_items.extend(self.items[start_index:])
        
        return visible_items
    
    def get_separator_display(self) -> str:
        """
        Get the display string/icon for the separator.
        
        Returns:
            Separator display string or icon class
        """
        if self.separator == 'custom':
            return self.separator_icon or '/'
        
        separator_map = {
            'arrow': '→',
            'slash': '/',
            'dot': '•'
        }
        
        return separator_map.get(self.separator, '/')
    
    def update_state(self, **kwargs) -> 'Breadcrumb':
        """
        Update component state with new values.
        
        Args:
            **kwargs: State updates
        
        Returns:
            Self for method chaining
        """
        if 'items' in kwargs:
            self.items = kwargs['items']
        
        if 'separator' in kwargs:
            self.separator = kwargs['separator']
        
        if 'separator_icon' in kwargs:
            self.separator_icon = kwargs['separator_icon']
        
        if 'show_home' in kwargs:
            self.show_home = kwargs['show_home']
        
        if 'home_url' in kwargs:
            self.home_url = kwargs['home_url']
        
        if 'home_icon' in kwargs:
            self.home_icon = kwargs['home_icon']
        
        if 'max_items' in kwargs:
            self.max_items = kwargs['max_items']
        
        if 'responsive' in kwargs:
            self.responsive = kwargs['responsive']
        
        if 'size' in kwargs:
            self.size = kwargs['size']
        
        if 'variant' in kwargs:
            self.variant = kwargs['variant']
        
        self._validate_component()
        return self
    
    def get_state(self) -> ComponentState:
        """
        Get the current component state.
        
        Returns:
            Current component state
        """
        return ComponentState(
            id=self.id,
            type='breadcrumb',
            props={
                'items': self.items,
                'separator': self.separator,
                'separator_icon': self.separator_icon,
                'show_home': self.show_home,
                'home_url': self.home_url,
                'home_icon': self.home_icon,
                'max_items': self.max_items,
                'responsive': self.responsive,
                'size': self.size,
                'variant': self.variant,
                'on_click': self.on_click
            },
            css_classes=self.get_css_classes(),
            validation=self._validate_component()
        )
    
    def get_css_classes(self) -> List[str]:
        """
        Get CSS classes for the component.
        
        Returns:
            List of CSS classes
        """
        classes = ['breadcrumb-container']
        
        if self.separator:
            classes.append(f'breadcrumb-separator-{self.separator}')
        
        if self.size:
            classes.append(f'breadcrumb-{self.size}')
        
        if self.variant:
            classes.append(f'breadcrumb-{self.variant}')
        
        if self.responsive:
            classes.append('breadcrumb-responsive')
        
        if self.show_home:
            classes.append('breadcrumb-has-home')
        
        return classes
    
    def compile(self) -> Dict[str, Any]:
        """
        Compile the component to a dictionary representation.
        
        Returns:
            Compiled component data
        """
        state = self.get_state()
        
        return {
            'type': 'breadcrumb',
            'id': self.id,
            'props': state.props,
            'css_classes': state.css_classes,
            'validation': {
                'is_valid': state.validation.is_valid,
                'errors': state.validation.errors
            }
        }


"""
Navigation Components Module
"""

from .menu.menu import Menu
from .navbar.navbar import Navbar
from .nav_item.nav_item import NavItem
from .tabs.tabs import Tabs
from .breadcrumb.breadcrumb import Breadcrumb

__all__ = [
    'Menu',
    'NavItem',
    'Navbar',
    'Tabs',
    'Breadcrumb'
]



from typing import Any, Dict
from ...base import Component

class Table(Component):
    """
    A table component with sorting, filtering, and pagination capabilities.
    """
    def __init__(
        self,
        columns,
        data_source,
        pagination = None,
        sorting = None,
        filters = None,
        **kwargs
    ) -> None:
        self.columns = columns
        self.data_source = data_source
        self.pagination = pagination or {"page_size": 10, "current": 1}
        self.sorting = sorting
        self.filters = filters
        self.extra_props = kwargs

    def render(self) -> Dict[str, Any]:
        return {
            "type": "Table",
            "props": {
                "columns": self.columns,
                "dataSource": self.data_source,
                "pagination": self.pagination,
                "sorting": self.sorting,
                "filters": self.filters,
                **self.extra_props
            }
        }



#!/usr/bin/env python3
"""
Comprehensive test script for Cacao framework package build validation.
This script tests for missing imports, packaging issues, and component availability.
"""

import sys
import os
import traceback
import importlib
import inspect
from pathlib import Path
import json
from datetime import datetime

# Test results tracking
test_results = {
    "timestamp": datetime.now().isoformat(),
    "python_version": sys.version,
    "passed": [],
    "failed": [],
    "warnings": [],
    "total_tests": 0,
    "passed_tests": 0,
    "failed_tests": 0
}

def log_test(test_name, status, message="", error=None):
    """Log test results"""
    test_results["total_tests"] += 1
    
    result = {
        "test": test_name,
        "status": status,
        "message": message,
        "error": str(error) if error else None
    }
    
    if status == "PASSED":
        test_results["passed"].append(result)
        test_results["passed_tests"] += 1
        print(f"✓ {test_name}: {message}")
    elif status == "FAILED":
        test_results["failed"].append(result)
        test_results["failed_tests"] += 1
        print(f"✗ {test_name}: {message}")
        if error:
            print(f"  Error: {error}")
    elif status == "WARNING":
        test_results["warnings"].append(result)
        print(f"⚠ {test_name}: {message}")

def test_basic_imports():
    """Test basic package imports"""
    print("\n=== Testing Basic Imports ===")
    
    # Test main package import
    try:
        import cacao
        log_test("Basic Import", "PASSED", "Successfully imported cacao")
    except Exception as e:
        log_test("Basic Import", "FAILED", "Failed to import cacao", e)
        return False
    
    # Test core modules
    core_modules = [
        'cacao.core',
        'cacao.core.app',
        'cacao.core.server',
        'cacao.core.state',
        'cacao.core.theme',
        'cacao.ui',
        'cacao.ui.types',
        'cacao.utilities',
        'cacao.cli',
        'cacao.extensions'
    ]
    
    for module_name in core_modules:
        try:
            importlib.import_module(module_name)
            log_test(f"Import {module_name}", "PASSED", f"Successfully imported {module_name}")
        except Exception as e:
            log_test(f"Import {module_name}", "FAILED", f"Failed to import {module_name}", e)
    
    return True

def test_component_imports():
    """Test UI component imports"""
    print("\n=== Testing Component Imports ===")
    
    # Test component package
    try:
        import cacao.ui.components
        log_test("Components Package", "PASSED", "Successfully imported cacao.ui.components")
    except Exception as e:
        log_test("Components Package", "FAILED", "Failed to import cacao.ui.components", e)
        return False
    
    # Test specific component categories
    component_categories = [
        'cacao.ui.components.data',
        'cacao.ui.components.forms',
        'cacao.ui.components.navigation',
        'cacao.ui.components.ui'
    ]
    
    for category in component_categories:
        try:
            importlib.import_module(category)
            log_test(f"Import {category}", "PASSED", f"Successfully imported {category}")
        except Exception as e:
            log_test(f"Import {category}", "FAILED", f"Failed to import {category}", e)
    
    return True

def test_static_assets():
    """Test static asset availability"""
    print("\n=== Testing Static Assets ===")
    
    try:
        import cacao
        cacao_path = Path(cacao.__file__).parent
        
        # Check for core static assets
        static_paths = [
            cacao_path / "core" / "static" / "js" / "cacao-core.js",
            cacao_path / "core" / "static" / "js" / "cacao-components.js",
            cacao_path / "core" / "static" / "css" / "icon-styles.css",
            cacao_path / "core" / "static" / "icons" / "cacao.svg",
            cacao_path / "core" / "static" / "index.html"
        ]
        
        for asset_path in static_paths:
            if asset_path.exists():
                log_test(f"Static Asset {asset_path.name}", "PASSED", f"Found {asset_path.name}")
            else:
                log_test(f"Static Asset {asset_path.name}", "FAILED", f"Missing {asset_path}")
        
        # Check for UI component assets
        ui_components_path = cacao_path / "ui" / "components"
        if ui_components_path.exists():
            js_files = list(ui_components_path.rglob("*.js"))
            css_files = list(ui_components_path.rglob("*.css"))
            
            if js_files:
                log_test("Component JS Files", "PASSED", f"Found {len(js_files)} JS files")
            else:
                log_test("Component JS Files", "WARNING", "No JS files found in components")
            
            if css_files:
                log_test("Component CSS Files", "PASSED", f"Found {len(css_files)} CSS files")
            else:
                log_test("Component CSS Files", "WARNING", "No CSS files found in components")
        else:
            log_test("UI Components Path", "FAILED", "UI components directory not found")
            
    except Exception as e:
        log_test("Static Assets", "FAILED", "Failed to check static assets", e)

def test_theme_imports():
    """Test theme imports"""
    print("\n=== Testing Theme Imports ===")
    
    try:
        import cacao.core.theme
        log_test("Theme Module", "PASSED", "Successfully imported cacao.core.theme")
        
        # Check if themes directory exists
        import cacao
        cacao_path = Path(cacao.__file__).parent
        themes_path = cacao_path / "ui" / "themes"
        
        if themes_path.exists():
            theme_files = list(themes_path.glob("*.py"))
            if theme_files:
                log_test("Theme Files", "PASSED", f"Found {len(theme_files)} theme files")
            else:
                log_test("Theme Files", "WARNING", "No theme files found")
        else:
            log_test("Themes Directory", "WARNING", "Themes directory not found")
            
    except Exception as e:
        log_test("Theme Imports", "FAILED", "Failed to test theme imports", e)

def test_cli_functionality():
    """Test CLI functionality"""
    print("\n=== Testing CLI Functionality ===")
    
    try:
        from cacao.cli import commands
        log_test("CLI Commands", "PASSED", "Successfully imported CLI commands")
        
        # Test CLI templates
        import cacao
        cacao_path = Path(cacao.__file__).parent
        templates_path = cacao_path / "cli" / "templates"
        
        if templates_path.exists():
            template_files = list(templates_path.rglob("*"))
            if template_files:
                log_test("CLI Templates", "PASSED", f"Found {len(template_files)} template files")
            else:
                log_test("CLI Templates", "WARNING", "No template files found")
        else:
            log_test("CLI Templates", "FAILED", "CLI templates directory not found")
            
    except Exception as e:
        log_test("CLI Functionality", "FAILED", "Failed to test CLI functionality", e)

def test_example_execution():
    """Test example script execution"""
    print("\n=== Testing Example Script Execution ===")
    
    # Test basic app creation
    try:
        import cacao
        app = cacao.App()
        log_test("App Creation", "PASSED", "Successfully created Cacao app instance")
        
        # Test basic state functionality
        from cacao.core.state import State
        state = State()
        log_test("State Creation", "PASSED", "Successfully created State instance")
        
    except Exception as e:
        log_test("Example Execution", "FAILED", "Failed to execute basic example", e)

def test_package_metadata():
    """Test package metadata"""
    print("\n=== Testing Package Metadata ===")
    
    try:
        import cacao
        
        # Check if package has version
        if hasattr(cacao, '__version__'):
            log_test("Package Version", "PASSED", f"Version: {cacao.__version__}")
        else:
            log_test("Package Version", "WARNING", "No version attribute found")
        
        # Check package path
        package_path = Path(cacao.__file__).parent
        log_test("Package Path", "PASSED", f"Located at: {package_path}")
        
        # Check package structure
        expected_dirs = ['core', 'ui', 'cli', 'extensions', 'utilities']
        missing_dirs = []
        
        for dir_name in expected_dirs:
            if not (package_path / dir_name).exists():
                missing_dirs.append(dir_name)
        
        if missing_dirs:
            log_test("Package Structure", "FAILED", f"Missing directories: {missing_dirs}")
        else:
            log_test("Package Structure", "PASSED", "All expected directories found")
            
    except Exception as e:
        log_test("Package Metadata", "FAILED", "Failed to check package metadata", e)

def test_extensions():
    """Test extensions functionality"""
    print("\n=== Testing Extensions ===")
    
    try:
        from cacao.extensions import react_extension
        log_test("React Extension", "PASSED", "Successfully imported react extension")
        
        # Test plugins
        from cacao.extensions.plugins import pandas_table
        log_test("Pandas Table Plugin", "PASSED", "Successfully imported pandas table plugin")
        
    except Exception as e:
        log_test("Extensions", "FAILED", "Failed to test extensions", e)

def save_test_results():
    """Save test results to file"""
    try:
        os.makedirs("test_results", exist_ok=True)
        
        with open("test_results/test_results.json", "w") as f:
            json.dump(test_results, f, indent=2)
        
        # Create summary report
        with open("test_results/test_summary.txt", "w") as f:
            f.write("CACAO PACKAGE TEST SUMMARY\n")
            f.write("=" * 30 + "\n\n")
            f.write(f"Timestamp: {test_results['timestamp']}\n")
            f.write(f"Python Version: {test_results['python_version']}\n\n")
            f.write(f"Total Tests: {test_results['total_tests']}\n")
            f.write(f"Passed: {test_results['passed_tests']}\n")
            f.write(f"Failed: {test_results['failed_tests']}\n")
            f.write(f"Warnings: {len(test_results['warnings'])}\n\n")
            
            if test_results['failed']:
                f.write("FAILED TESTS:\n")
                for failure in test_results['failed']:
                    f.write(f"- {failure['test']}: {failure['message']}\n")
                    if failure['error']:
                        f.write(f"  Error: {failure['error']}\n")
                f.write("\n")
            
            if test_results['warnings']:
                f.write("WARNINGS:\n")
                for warning in test_results['warnings']:
                    f.write(f"- {warning['test']}: {warning['message']}\n")
                f.write("\n")
        
        print(f"\nTest results saved to test_results/")
        
    except Exception as e:
        print(f"Failed to save test results: {e}")

def main():
    """Main test runner"""
    print("CACAO PACKAGE BUILD VALIDATION")
    print("=" * 40)
    print(f"Python version: {sys.version}")
    print(f"Test mode: {os.environ.get('CACAO_TEST_MODE', 'false')}")
    print()
    
    # Run all tests
    test_basic_imports()
    test_component_imports()
    test_static_assets()
    test_theme_imports()
    test_cli_functionality()
    test_example_execution()
    test_package_metadata()
    test_extensions()
    
    # Print summary
    print("\n" + "=" * 40)
    print("TEST SUMMARY")
    print("=" * 40)
    print(f"Total tests: {test_results['total_tests']}")
    print(f"Passed: {test_results['passed_tests']}")
    print(f"Failed: {test_results['failed_tests']}")
    print(f"Warnings: {len(test_results['warnings'])}")
    
    if test_results['failed_tests'] > 0:
        print("\nFAILED TESTS:")
        for failure in test_results['failed']:
            print(f"- {failure['test']}: {failure['message']}")
    
    if test_results['warnings']:
        print("\nWARNINGS:")
        for warning in test_results['warnings']:
            print(f"- {warning['test']}: {warning['message']}")
    
    # Save results
    save_test_results()
    
    # Exit with appropriate code
    if test_results['failed_tests'] > 0:
        print("\n❌ Some tests failed. Package may have issues.")
        sys.exit(1)
    else:
        print("\n✅ All tests passed! Package is ready for distribution.")
        sys.exit(0)

if __name__ == "__main__":
    main()


"""
Tabs Component

A tab navigation component that allows users to switch between different content panels.
Supports horizontal and vertical orientations, various sizes, and customizable styling.
"""

from typing import List, Dict, Any, Optional, Union
from cacao.ui.components.base import Component
from cacao.ui.types import ComponentState, ValidationResult


class Tabs(Component):
    """
    A tab navigation component for switching between content panels.
    
    Features:
    - Multiple tab items with content panels
    - Horizontal and vertical orientations
    - Various sizes and styling options
    - Keyboard navigation support
    - Disabled tabs
    - Custom icons and badges
    - Closable tabs
    - Animated transitions
    """
    
    def __init__(
        self,
        id: str,
        items: List[Dict[str, Any]] = None,
        active_key: Optional[str] = None,
        orientation: str = 'horizontal',
        size: str = 'medium',
        variant: str = 'default',
        animated: bool = True,
        closable: bool = False,
        centered: bool = False,
        show_add_button: bool = False,
        max_width: Optional[str] = None,
        on_change: Optional[str] = None,
        on_close: Optional[str] = None,
        on_add: Optional[str] = None,
        **kwargs
    ):
        """
        Initialize a Tabs component.
        
        Args:
            id: Unique identifier for the component
            items: List of tab items with keys, labels, and content
            active_key: Key of the currently active tab
            orientation: Tab orientation ('horizontal' or 'vertical')
            size: Tab size ('small', 'medium', 'large')
            variant: Tab variant ('default', 'card', 'pill', 'underline')
            animated: Whether to animate tab transitions
            closable: Whether tabs can be closed
            centered: Whether to center the tabs
            show_add_button: Whether to show an add button
            max_width: Maximum width for the tab container
            on_change: JavaScript function to call when tab changes
            on_close: JavaScript function to call when tab closes
            on_add: JavaScript function to call when add button is clicked
        """
        super().__init__(id, **kwargs)
        
        self.items = items or []
        self.active_key = active_key
        self.orientation = orientation
        self.size = size
        self.variant = variant
        self.animated = animated
        self.closable = closable
        self.centered = centered
        self.show_add_button = show_add_button
        self.max_width = max_width
        self.on_change = on_change
        self.on_close = on_close
        self.on_add = on_add
        
        # Set default active key if not provided
        if not self.active_key and self.items:
            self.active_key = self.items[0].get('key')
        
        # Validate component
        self._validate_component()
    
    def _validate_component(self) -> ValidationResult:
        """Validate the tabs component configuration."""
        errors = []
        
        # Validate orientation
        valid_orientations = ['horizontal', 'vertical']
        if self.orientation not in valid_orientations:
            errors.append(f"Invalid orientation: {self.orientation}. Must be one of: {valid_orientations}")
        
        # Validate size
        valid_sizes = ['small', 'medium', 'large']
        if self.size not in valid_sizes:
            errors.append(f"Invalid size: {self.size}. Must be one of: {valid_sizes}")
        
        # Validate variant
        valid_variants = ['default', 'card', 'pill', 'underline']
        if self.variant not in valid_variants:
            errors.append(f"Invalid variant: {self.variant}. Must be one of: {valid_variants}")
        
        # Validate items
        if not self.items:
            errors.append("Tabs must have at least one item")
        else:
            for i, item in enumerate(self.items):
                if not isinstance(item, dict):
                    errors.append(f"Item {i} must be a dictionary")
                    continue
                
                if 'key' not in item:
                    errors.append(f"Item {i} must have a 'key' field")
                
                if 'label' not in item:
                    errors.append(f"Item {i} must have a 'label' field")
        
        # Validate active key exists
        if self.active_key:
            item_keys = [item.get('key') for item in self.items]
            if self.active_key not in item_keys:
                errors.append(f"Active key '{self.active_key}' not found in items")
        
        return ValidationResult(is_valid=len(errors) == 0, errors=errors)
    
    def add_item(self, key: str, label: str, content: str = '', icon: str = None, 
                 badge: Union[str, int] = None, disabled: bool = False, 
                 closable: bool = None) -> 'Tabs':
        """
        Add a new tab item.
        
        Args:
            key: Unique key for the tab
            label: Display label for the tab
            content: Content to display in the tab panel
            icon: Icon class for the tab
            badge: Badge content to display
            disabled: Whether the tab is disabled
            closable: Whether this specific tab can be closed
        
        Returns:
            Self for method chaining
        """
        item = {
            'key': key,
            'label': label,
            'content': content,
            'disabled': disabled
        }
        
        if icon:
            item['icon'] = icon
        
        if badge is not None:
            item['badge'] = badge
        
        if closable is not None:
            item['closable'] = closable
        
        self.items.append(item)
        
        # Set as active if it's the first item
        if len(self.items) == 1:
            self.active_key = key
        
        return self
    
    def remove_item(self, key: str) -> 'Tabs':
        """
        Remove a tab item by key.
        
        Args:
            key: Key of the tab to remove
        
        Returns:
            Self for method chaining
        """
        self.items = [item for item in self.items if item.get('key') != key]
        
        # Update active key if removed item was active
        if self.active_key == key and self.items:
            self.active_key = self.items[0].get('key')
        elif not self.items:
            self.active_key = None
        
        return self
    
    def set_active(self, key: str) -> 'Tabs':
        """
        Set the active tab by key.
        
        Args:
            key: Key of the tab to make active
        
        Returns:
            Self for method chaining
        """
        item_keys = [item.get('key') for item in self.items]
        if key in item_keys:
            self.active_key = key
        
        return self
    
    def disable_item(self, key: str, disabled: bool = True) -> 'Tabs':
        """
        Enable or disable a tab item.
        
        Args:
            key: Key of the tab to modify
            disabled: Whether to disable the tab
        
        Returns:
            Self for method chaining
        """
        for item in self.items:
            if item.get('key') == key:
                item['disabled'] = disabled
                break
        
        return self
    
    def get_active_item(self) -> Optional[Dict[str, Any]]:
        """
        Get the currently active tab item.
        
        Returns:
            Active tab item or None if no active tab
        """
        if not self.active_key:
            return None
        
        for item in self.items:
            if item.get('key') == self.active_key:
                return item
        
        return None
    
    def get_enabled_items(self) -> List[Dict[str, Any]]:
        """
        Get all enabled tab items.
        
        Returns:
            List of enabled tab items
        """
        return [item for item in self.items if not item.get('disabled', False)]
    
    def update_state(self, **kwargs) -> 'Tabs':
        """
        Update component state with new values.
        
        Args:
            **kwargs: State updates
        
        Returns:
            Self for method chaining
        """
        if 'active_key' in kwargs:
            self.set_active(kwargs['active_key'])
        
        if 'items' in kwargs:
            self.items = kwargs['items']
        
        if 'orientation' in kwargs:
            self.orientation = kwargs['orientation']
        
        if 'size' in kwargs:
            self.size = kwargs['size']
        
        if 'variant' in kwargs:
            self.variant = kwargs['variant']
        
        if 'animated' in kwargs:
            self.animated = kwargs['animated']
        
        if 'closable' in kwargs:
            self.closable = kwargs['closable']
        
        if 'centered' in kwargs:
            self.centered = kwargs['centered']
        
        if 'show_add_button' in kwargs:
            self.show_add_button = kwargs['show_add_button']
        
        self._validate_component()
        return self
    
    def get_state(self) -> ComponentState:
        """
        Get the current component state.
        
        Returns:
            Current component state
        """
        return ComponentState(
            id=self.id,
            type='tabs',
            props={
                'items': self.items,
                'active_key': self.active_key,
                'orientation': self.orientation,
                'size': self.size,
                'variant': self.variant,
                'animated': self.animated,
                'closable': self.closable,
                'centered': self.centered,
                'show_add_button': self.show_add_button,
                'max_width': self.max_width,
                'on_change': self.on_change,
                'on_close': self.on_close,
                'on_add': self.on_add
            },
            css_classes=self.get_css_classes(),
            validation=self._validate_component()
        )
    
    def get_css_classes(self) -> List[str]:
        """
        Get CSS classes for the component.
        
        Returns:
            List of CSS classes
        """
        classes = ['tabs-container']
        
        if self.orientation:
            classes.append(f'tabs-{self.orientation}')
        
        if self.size:
            classes.append(f'tabs-{self.size}')
        
        if self.variant:
            classes.append(f'tabs-{self.variant}')
        
        if self.animated:
            classes.append('tabs-animated')
        
        if self.closable:
            classes.append('tabs-closable')
        
        if self.centered:
            classes.append('tabs-centered')
        
        if self.show_add_button:
            classes.append('tabs-has-add-button')
        
        return classes
    
    def compile(self) -> Dict[str, Any]:
        """
        Compile the component to a dictionary representation.
        
        Returns:
            Compiled component data
        """
        state = self.get_state()
        
        return {
            'type': 'tabs',
            'id': self.id,
            'props': state.props,
            'css_classes': state.css_classes,
            'validation': {
                'is_valid': state.validation.is_valid,
                'errors': state.validation.errors
            }
        }


"""
Logging mixin to record activity, such as state changes and user interactions.
"""

from datetime import datetime
import colorama
import sys

# Import the ASCII debug setting
sys.path.append('c:\\Users\\Juan\\Documents\\GitHub\\Cacao\\cacao\\core')
try:
    from app import ASCII_DEBUG_MODE
except ImportError:
    # Default to False if import fails
    ASCII_DEBUG_MODE = False

# Initialize colorama for Windows support
colorama.init()

class Colors:
    HEADER = '\033[95m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    RED = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'

class LoggingMixin:
    """
    A mixin that provides logging functionality for tracking
    component or application events with different log levels and formatting.
    """

    def log_event(self, message: str) -> None:
        """
        Logs a timestamped event message.
        
        Args:
            message: The message to log.
        """
        now = datetime.utcnow().isoformat()
        print(f"[{now}] [CacaoLog] {message}")

    def log(self, message: str, level: str = "info", emoji: str = "") -> None:
        """
        Logs a formatted message with timestamp, color, and emoji.
        
        Args:
            message: The message to log.
            level: Log level (debug, info, warning, error).
            emoji: Optional emoji to prefix the message.
        """
        timestamp = datetime.now().strftime("%H:%M:%S")
        color = {
            "debug": Colors.BLUE,
            "info": Colors.GREEN,
            "warning": Colors.YELLOW,
            "error": Colors.RED
        }.get(level, Colors.ENDC)
        
        # Set the display emoji
        display_emoji = ""
        if emoji:
            if ASCII_DEBUG_MODE:
                # When ASCII debug mode is ON, replace emojis with ASCII alternatives
                emoji_replacements = {
                    "🍫": "C", "🌎": "W", "🔌": "*", "📡": "*", "👀": "*", 
                    "🔄": "*", "🌟": "*", "📂": "*", "🎯": "*", "🕒": "*",
                    "🔢": "*", "⚠️": "!", "❌": "X", "💥": "!", "👋": "*",
                    "📢": "*", "🔥": "*", "❓": "?", "⏰": "*", "🔁": "*",
                }
                
                # If emoji isn't in our mapping, default to "*"
                display_emoji = emoji_replacements.get(emoji, "*")
            else:
                # When ASCII debug mode is OFF, use the actual emoji
                display_emoji = emoji
                
        formatted_message = f"{color}{timestamp} {display_emoji}  {message}{Colors.ENDC}"
        print(formatted_message)


"""
Upload Component Python Integration
"""

from typing import Dict, Any, Optional, List
from ...base import Component

class Upload(Component):
    """
    Server-side logic for upload component
    """
    
    def __init__(self, multiple: bool = False, disabled: bool = False, 
                 accept: str = None, max_size: int = None, **kwargs):
        """
        Initialize upload component
        
        Args:
            multiple: Whether to allow multiple file uploads
            disabled: Whether upload is disabled
            accept: Accepted file types (e.g., ".jpg,.png,.pdf")
            max_size: Maximum file size in bytes
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.multiple = multiple
        self.disabled = disabled
        self.accept = accept
        self.max_size = max_size
        self.uploaded_files = []
    
    def validate_file(self, file_info: Dict[str, Any]) -> Dict[str, Any]:
        """
        Validate a single file
        
        Args:
            file_info: File information dictionary
            
        Returns:
            Dict containing validation result
        """
        errors = []
        
        # Check file size
        if self.max_size and file_info.get('size', 0) > self.max_size:
            errors.append(f"File size exceeds maximum of {self.max_size} bytes")
        
        # Check file type
        if self.accept:
            file_name = file_info.get('name', '')
            file_ext = '.' + file_name.split('.')[-1] if '.' in file_name else ''
            accepted_types = [t.strip() for t in self.accept.split(',')]
            
            if file_ext not in accepted_types:
                errors.append(f"File type {file_ext} not accepted. Accepted types: {self.accept}")
        
        return {
            "valid": len(errors) == 0,
            "errors": errors
        }
    
    def on_upload(self, files: List[Dict[str, Any]]) -> Dict[str, Any]:
        """
        Handle file upload
        
        Args:
            files: List of file information dictionaries
            
        Returns:
            Dict containing response data
        """
        if not self.multiple and len(files) > 1:
            return {
                "success": False,
                "error": "Multiple files not allowed",
                "files": []
            }
        
        validated_files = []
        upload_errors = []
        
        for file_info in files:
            validation = self.validate_file(file_info)
            
            if validation["valid"]:
                validated_files.append({
                    **file_info,
                    "status": "uploaded",
                    "upload_time": "now"  # Replace with actual timestamp
                })
            else:
                upload_errors.extend(validation["errors"])
        
        # Update uploaded files list
        if self.multiple:
            self.uploaded_files.extend(validated_files)
        else:
            self.uploaded_files = validated_files
        
        return {
            "success": len(upload_errors) == 0,
            "files": validated_files,
            "errors": upload_errors,
            "total_files": len(self.uploaded_files)
        }
    
    def remove_file(self, file_id: str) -> Dict[str, Any]:
        """
        Remove an uploaded file
        
        Args:
            file_id: ID of file to remove
            
        Returns:
            Dict containing response data
        """
        original_count = len(self.uploaded_files)
        self.uploaded_files = [f for f in self.uploaded_files if f.get('id') != file_id]
        
        return {
            "success": len(self.uploaded_files) < original_count,
            "removed": original_count - len(self.uploaded_files) > 0,
            "remaining_files": len(self.uploaded_files)
        }
    
    def clear_files(self) -> Dict[str, Any]:
        """
        Clear all uploaded files
        
        Returns:
            Dict containing response data
        """
        cleared_count = len(self.uploaded_files)
        self.uploaded_files = []
        
        return {
            "success": True,
            "cleared_count": cleared_count,
            "remaining_files": 0
        }
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        props = {
            "multiple": self.multiple,
            "disabled": self.disabled,
            **self.get_base_props()
        }
        
        if self.accept:
            props["accept"] = self.accept
        if self.max_size:
            props["maxSize"] = self.max_size
            
        return {
            "type": "upload",
            "props": props
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Upload':
        """
        Create upload component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Upload instance
        """
        return Upload(
            multiple=data.get('multiple', False),
            disabled=data.get('disabled', False),
            accept=data.get('accept'),
            max_size=data.get('maxSize')
        )

def create_upload(multiple: bool = False, disabled: bool = False, accept: str = None,
                 max_size: int = None, **kwargs) -> Dict[str, Any]:
    """Create an upload component"""
    component = Upload(multiple, disabled, accept, max_size, **kwargs)
    return component.to_dict()


"""
Timepicker Component Python Integration
"""

from typing import Dict, Any, Optional
from datetime import datetime, time
from ...base import Component

class Timepicker(Component):
    """
    Server-side logic for timepicker component
    """
    
    def __init__(self, value: str = None, disabled: bool = False, min_time: str = None,
                 max_time: str = None, **kwargs):
        """
        Initialize timepicker component
        
        Args:
            value: Initial time value (HH:MM format)
            disabled: Whether timepicker is disabled
            min_time: Minimum selectable time
            max_time: Maximum selectable time
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.value = value
        self.disabled = disabled
        self.min_time = min_time
        self.max_time = max_time
    
    def validate(self) -> bool:
        """
        Validate time value
        
        Returns:
            bool: True if validation passes
        """
        if not self.value:
            return True
            
        try:
            # Parse the time to ensure it's valid
            parsed_time = datetime.strptime(self.value, '%H:%M').time()
            
            # Check min/max constraints
            if self.min_time:
                min_parsed = datetime.strptime(self.min_time, '%H:%M').time()
                if parsed_time < min_parsed:
                    return False
                    
            if self.max_time:
                max_parsed = datetime.strptime(self.max_time, '%H:%M').time()
                if parsed_time > max_parsed:
                    return False
                    
            return True
        except ValueError:
            return False
    
    def on_change(self, value: str) -> Dict[str, Any]:
        """
        Handle time value change
        
        Args:
            value: New time value
            
        Returns:
            Dict containing response data
        """
        old_value = self.value
        self.value = value
        is_valid = self.validate()
        
        # Parse time for additional info
        parsed_time = None
        if value and is_valid:
            try:
                parsed_time = datetime.strptime(value, '%H:%M').time()
            except ValueError:
                pass
        
        return {
            "success": True,
            "value": self.value,
            "previousValue": old_value,
            "valid": is_valid,
            "parsedTime": {
                "hour": parsed_time.hour if parsed_time else None,
                "minute": parsed_time.minute if parsed_time else None,
                "hour12": parsed_time.hour % 12 if parsed_time else None,
                "ampm": "AM" if parsed_time and parsed_time.hour < 12 else "PM" if parsed_time else None
            } if parsed_time else None
        }
    
    def set_to_now(self) -> Dict[str, Any]:
        """
        Set time to current time
        
        Returns:
            Dict containing response data
        """
        now = datetime.now().time().strftime('%H:%M')
        return self.on_change(now)
    
    def clear_time(self) -> Dict[str, Any]:
        """
        Clear time value
        
        Returns:
            Dict containing response data
        """
        return self.on_change("")
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        props = {
            "disabled": self.disabled,
            **self.get_base_props()
        }
        
        if self.value:
            props["value"] = self.value
        if self.min_time:
            props["min"] = self.min_time
        if self.max_time:
            props["max"] = self.max_time
            
        return {
            "type": "timepicker",
            "props": props
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Timepicker':
        """
        Create timepicker component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Timepicker instance
        """
        return Timepicker(
            value=data.get('value'),
            disabled=data.get('disabled', False),
            min_time=data.get('min'),
            max_time=data.get('max')
        )

def create_timepicker(value: str = None, disabled: bool = False, min_time: str = None,
                     max_time: str = None, **kwargs) -> Dict[str, Any]:
    """Create a timepicker component"""
    component = Timepicker(value, disabled, min_time, max_time, **kwargs)
    return component.to_dict()


"""
Slider Component Python Integration
"""

from typing import Dict, Any, Optional
from ...base import Component

class Slider(Component):
    """
    Server-side logic for slider component
    """
    
    def __init__(self, min_val: float = 0, max_val: float = 100, step: float = 1,
                 value: float = 0, onChange: Dict[str, Any] = None, **kwargs):
        """
        Initialize slider component
        
        Args:
            min_val: Minimum value
            max_val: Maximum value
            step: Step increment
            value: Current value
            onChange: Change handler configuration
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.min_val = min_val
        self.max_val = max_val
        self.step = step
        self.value = value
        self.onChange = onChange
    
    def validate(self) -> bool:
        """
        Validate slider value
        
        Returns:
            bool: True if validation passes
        """
        # Check if value is within bounds
        if self.value < self.min_val or self.value > self.max_val:
            return False
            
        # Check if value aligns with step increment
        if self.step > 0:
            steps_from_min = (self.value - self.min_val) / self.step
            return abs(steps_from_min - round(steps_from_min)) < 1e-10
            
        return True
    
    def on_change(self, value: float) -> Dict[str, Any]:
        """
        Handle slider value change
        
        Args:
            value: New slider value
            
        Returns:
            Dict containing response data
        """
        old_value = self.value
        self.value = value
        is_valid = self.validate()
        
        # Calculate percentage
        percentage = ((value - self.min_val) / (self.max_val - self.min_val)) * 100 if self.max_val != self.min_val else 0
        
        return {
            "success": True,
            "value": self.value,
            "previousValue": old_value,
            "valid": is_valid,
            "percentage": percentage,
            "min": self.min_val,
            "max": self.max_val
        }
    
    def increment(self) -> Dict[str, Any]:
        """
        Increment slider value by step
        
        Returns:
            Dict containing response data
        """
        new_value = min(self.value + self.step, self.max_val)
        return self.on_change(new_value)
    
    def decrement(self) -> Dict[str, Any]:
        """
        Decrement slider value by step
        
        Returns:
            Dict containing response data
        """
        new_value = max(self.value - self.step, self.min_val)
        return self.on_change(new_value)
    
    def set_to_min(self) -> Dict[str, Any]:
        """
        Set slider to minimum value
        
        Returns:
            Dict containing response data
        """
        return self.on_change(self.min_val)
    
    def set_to_max(self) -> Dict[str, Any]:
        """
        Set slider to maximum value
        
        Returns:
            Dict containing response data
        """
        return self.on_change(self.max_val)
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        props = {
            "min": self.min_val,
            "max": self.max_val,
            "step": self.step,
            "value": self.value,
            **self.get_base_props()
        }
        
        if self.onChange:
            props["onChange"] = self.onChange
            
        return {
            "type": "slider",
            "props": props
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Slider':
        """
        Create slider component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Slider instance
        """
        return Slider(
            min_val=data.get('min', 0),
            max_val=data.get('max', 100),
            step=data.get('step', 1),
            value=data.get('value', 0),
            onChange=data.get('onChange')
        )

def create_slider(min_val: float = 0, max_val: float = 100, step: float = 1,
                 value: float = 0, onChange: Dict[str, Any] = None, **kwargs) -> Dict[str, Any]:
    """Create a slider component"""
    component = Slider(min_val, max_val, step, value, onChange, **kwargs)
    return component.to_dict()


"""
Custom theme module for the Cacao framework.
Defines a custom theme to override default styles and behaviors.
"""

class CustomTheme:
    """
    Custom theme for Cacao. Override default styling and behavior.
    """
    CSS = """
    :root {
        --primary-color: #2ecc71;
        --font-family: 'Roboto', sans-serif;
    }
    body {
        font-family: 'Roboto', sans-serif;
    }
    """

    JS = """
    document.addEventListener('cacao-init', () => {
        console.log('Custom theme loaded!');
    });
    """



"""
Command registry for the Cacao CLI.
Defines available CLI commands and their handlers.
"""

import argparse
import sys
import os
from pathlib import Path
from typing import Callable, Dict, List

COMMANDS: Dict[str, Callable] = {}

def register_command(name: str) -> Callable:
    """
    Decorator to register a new CLI command.
    """
    def decorator(func: Callable) -> Callable:
        COMMANDS[name] = func
        return func
    return decorator

@register_command("serve")
def serve_command(args: List[str]) -> None:
    """
    Run the development server with hot reload.
    """
    parser = argparse.ArgumentParser(description="Run the Cacao development server")
    parser.add_argument("-v", "--verbose", action="store_true", help="Enable verbose logging")
    parser.add_argument("--host", default="localhost", help="Host to bind the server to")
    parser.add_argument("--port", type=int, default=1634, help="Port for the HTTP server")
    parser.add_argument("--ws-port", type=int, default=1633, help="Port for the WebSocket server")
    parser.add_argument("--pwa", action="store_true", help="Enable Progressive Web App mode")
    
    parsed_args = parser.parse_args(args)
    
    try:
        from cacao.core.server import CacaoServer
        server = CacaoServer(
            host=parsed_args.host,
            http_port=parsed_args.port,
            ws_port=parsed_args.ws_port,
            verbose=parsed_args.verbose,
            enable_pwa=parsed_args.pwa
        )
        print(f"Starting Cacao development server...")
        server.run()
    except ImportError:
        print("Error: Could not import CacaoServer. Make sure Cacao is installed correctly.")
        sys.exit(1)
    except KeyboardInterrupt:
        print("\nServer stopped by user")
        sys.exit(0)
    except Exception as e:
        print(f"Error starting server: {str(e)}")
        sys.exit(1)
        
@register_command("build-components")
def build_components_command(args: List[str]) -> None:
    """
    Build component JavaScript files into cacao-components.js
    """
    parser = argparse.ArgumentParser(description="Compile modular components into cacao-components.js")
    parser.add_argument('--components-dir', default='cacao/ui/components',
                        help='Directory containing component directories')
    parser.add_argument('--output', default='cacao/core/static/js/cacao-components.js',
                        help='Output path for compiled components file')
    parser.add_argument('--force', action='store_true',
                        help='Force rebuild even if files haven\'t changed')
    parser.add_argument('--verbose', action='store_true',
                        help='Show detailed compilation information')
    
    parsed_args = parser.parse_args(args)
    
    try:
        from cacao.core.component_compiler import compile_components
        
        print(f"🔧 Building components from {parsed_args.components_dir}...")
        print("📝 Automatic function call transformation enabled")
        print("   Direct CacaoCore function calls will be auto-namespaced")
        print("   See docs/COMPONENT_DEVELOPMENT_GUIDE.md for best practices")
        print()
        
        success = compile_components(
            components_dir=parsed_args.components_dir,
            output_path=parsed_args.output,
            force=parsed_args.force,
            verbose=parsed_args.verbose
        )
        
        if success:
            print(f"✅ Component compilation completed successfully")
            print(f"📁 Output: {parsed_args.output}")
        else:
            print(f"❌ Component compilation failed")
            sys.exit(1)
            
    except ImportError as e:
        print(f"Error: Could not import component compiler: {e}")
        sys.exit(1)
    except Exception as e:
        print(f"Error during compilation: {e}")
        sys.exit(1)


def run_cli() -> None:
    """
    Parse arguments and execute the corresponding CLI command.
    """
    parser = argparse.ArgumentParser(description="Cacao CLI")
    parser.add_argument("command", help="Command to run")
    args, unknown = parser.parse_known_args()

    command_func = COMMANDS.get(args.command)
    if command_func:
        command_func(unknown)
    else:
        print(f"Command '{args.command}' not recognized.")
        print(f"Available commands: {', '.join(COMMANDS.keys())}")



"""
Badge component for adding small indicators to elements.
"""

from typing import Any, Dict
from ...base import Component


class Badge(Component):
    """
    Adds a small indicator (e.g., a number or dot) to an element.
    """
    def __init__(
        self,
        count = None,
        children = None,
        dot = False,
        show_zero = False,
        **kwargs
    ) -> None:
        self.count = count
        self.children = children
        self.dot = dot
        self.show_zero = show_zero
        self.extra_props = kwargs

    def render(self) -> Dict[str, Any]:
        return {
            "type": "badge",
            "props": {
                "count": self.count,
                "children": self.children.render() if self.children else None,
                "dot": self.dot,
                "showZero": self.show_zero,
                **self.extra_props
            }
        }


"""
Validation mixin for the Cacao framework.
Provides input sanitization to ensure data integrity and security.
"""

import re
from typing import Any, Callable

class ValidationMixin:
    """
    Mixin that provides methods for sanitizing and validating user input.
    """

    @staticmethod
    def sanitize(input_data: str) -> str:
        """
        Sanitize a given string by escaping potentially harmful characters.
        This implementation removes script tags and escapes HTML special characters.
        """
        # Remove script tags and their content
        sanitized = re.sub(
            r'<script.*?>.*?</script>',
            '',
            input_data,
            flags=re.IGNORECASE | re.DOTALL
        )
        # Escape HTML special characters
        sanitized = (
            sanitized.replace("&", "&amp;")
                     .replace("<", "&lt;")
                     .replace(">", "&gt;")
                     .replace("\"", "&quot;")
                     .replace("'", "&#x27;")
        )
        return sanitized

    @staticmethod
    def validate_length(input_data: str, min_length: int = 1, max_length: int = 1000) -> bool:
        """
        Validates that the length of input_data is within specified bounds.
        """
        return min_length <= len(input_data) <= max_length

    def validate(self, input_data: Any, validator: Callable[[Any], bool]) -> bool:
        """
        General validation method that applies a validator function to the input_data.
        """
        return validator(input_data)



"""
Sidebar theme module for the Cacao framework.
Defines theme variables specifically for the sidebar layout component.
"""

from typing import Dict, Any, Optional

class SidebarTheme:
    """
    Theme configuration for the SidebarLayout component.
    Provides color and style variables for customizing the sidebar appearance.
    """
    
    # Default color palette
    DEFAULT_COLORS = {
        # Background colors
        "content_bg": "#FAF6F3",          # Light cream background for content area
        "sidebar_bg": "#2D2013",          # Dark brown background for sidebar
        "sidebar_header_bg": "#6B4226",   # Cacao brown header
        "card_bg": "#FFFFFF",             # White background for content cards
        
        # Text colors
        "title_color": "#6B4226",         # Cacao brown for titles
        "text_color": "#2D2013",          # Dark brown for text
        "sidebar_text": "#D6C3B6",        # Light brown text for sidebar
        "active_text": "#FFFFFF",         # White text for active items
        "app_title_color": "#D6C3B6",     # Light brown text for app title
        
        # Border colors
        "border_color": "#D6C3B6",        # Light brown border
        "card_border": "#E6D7CC",         # Very light brown border for cards
        "sidebar_border": "#503418",      # Medium brown border for sidebar sections
        
        # Interactive element colors
        "active_bg": "#D6C3B6",           # Light brown active background (was Cacao brown)
        "active_icon_bg": "#8B5E41",      # Slightly lighter brown for active icons
        "inactive_icon_bg": "rgba(107, 66, 38, 0.3)",  # Transparent brown for inactive icons
    }
    
    # Default spacing and sizing
    DEFAULT_SPACING = {
        "sidebar_width": "250px",         # Width of expanded sidebar
        "sidebar_collapsed_width": "64px", # Width of collapsed sidebar
        "content_padding": "24px 32px",   # Padding for content area
        "card_padding": "24px",           # Padding for content cards
        "nav_item_padding": "12px 16px",  # Padding for navigation items
    }
    
    # Default font settings
    DEFAULT_FONTS = {
        "title_size": "24px",             # Size for page titles
        "title_weight": "700",            # Font weight for titles
        "nav_item_size": "15px",          # Size for navigation items
        "nav_item_weight": "500",         # Font weight for navigation items
    }
    
    def __init__(self, 
                 colors: Optional[Dict[str, str]] = None,
                 spacing: Optional[Dict[str, str]] = None,
                 fonts: Optional[Dict[str, str]] = None):
        """
        Initialize a sidebar theme with customizable properties.
        
        Args:
            colors: Optional dictionary of color overrides
            spacing: Optional dictionary of spacing overrides
            fonts: Optional dictionary of font setting overrides
        """
        # Merge provided values with defaults
        self.colors = {**self.DEFAULT_COLORS, **(colors or {})}
        self.spacing = {**self.DEFAULT_SPACING, **(spacing or {})}
        self.fonts = {**self.DEFAULT_FONTS, **(fonts or {})}
    
    def get_all(self) -> Dict[str, Dict[str, str]]:
        """
        Get all theme properties as a dictionary.
        
        Returns:
            Dictionary containing all theme properties
        """
        return {
            "colors": self.colors,
            "spacing": self.spacing,
            "fonts": self.fonts
        }
    
    def get_css_variables(self) -> str:
        """
        Generate CSS variables for this theme.
        
        Returns:
            CSS string with variable definitions
        """
        css = ":root {\n"
        
        # Add color variables
        for key, value in self.colors.items():
            css += f"  --sidebar-{key}: {value};\n"
            
        # Add spacing variables
        for key, value in self.spacing.items():
            css += f"  --sidebar-{key}: {value};\n"
            
        # Add font variables
        for key, value in self.fonts.items():
            css += f"  --sidebar-{key}: {value};\n"
            
        css += "}\n"
        return css


"""
Text Component Python Integration
"""

from typing import Dict, Any, Optional
from ...base import Component

class Text(Component):
    """
    Server-side logic for text component
    """
    
    def __init__(self, text: str = "", element_type: str = "span", 
                 size: str = "medium", weight: str = "normal", 
                 color: str = "default", italic: bool = False, 
                 underline: bool = False, strikethrough: bool = False, 
                 **kwargs):
        """
        Initialize text component
        
        Args:
            text: Text content
            element_type: HTML element type (span, p, h1, h2, h3, h4, h5, h6, div)
            size: Text size (small, medium, large, extra-large)
            weight: Font weight (light, normal, bold, extra-bold)
            color: Text color (default, primary, secondary, success, warning, danger, info)
            italic: Whether text is italic
            underline: Whether text is underlined
            strikethrough: Whether text has strikethrough
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.text = text
        self.element_type = element_type
        self.size = size
        self.weight = weight
        self.color = color
        self.italic = italic
        self.underline = underline
        self.strikethrough = strikethrough
    
    def validate(self) -> bool:
        """
        Validate text configuration
        
        Returns:
            bool: True if validation passes
        """
        # Validate element type
        valid_elements = ["span", "p", "h1", "h2", "h3", "h4", "h5", "h6", "div"]
        if self.element_type not in valid_elements:
            return False
            
        # Validate size
        valid_sizes = ["small", "medium", "large", "extra-large"]
        if self.size not in valid_sizes:
            return False
            
        # Validate weight
        valid_weights = ["light", "normal", "bold", "extra-bold"]
        if self.weight not in valid_weights:
            return False
            
        # Validate color
        valid_colors = ["default", "primary", "secondary", "success", "warning", "danger", "info"]
        if self.color not in valid_colors:
            return False
            
        return True
    
    def set_text(self, text: str) -> Dict[str, Any]:
        """
        Set text content
        
        Args:
            text: New text content
            
        Returns:
            Dict containing response data
        """
        old_text = self.text
        self.text = text
        
        return {
            "success": True,
            "text": self.text,
            "previous_text": old_text
        }
    
    def set_style(self, size: str = None, weight: str = None, 
                  color: str = None, italic: bool = None, 
                  underline: bool = None, strikethrough: bool = None) -> Dict[str, Any]:
        """
        Set text style properties
        
        Args:
            size: Text size
            weight: Font weight
            color: Text color
            italic: Italic state
            underline: Underline state
            strikethrough: Strikethrough state
            
        Returns:
            Dict containing response data
        """
        changes = {}
        
        if size is not None:
            valid_sizes = ["small", "medium", "large", "extra-large"]
            if size not in valid_sizes:
                return {
                    "success": False,
                    "message": f"Invalid size. Valid sizes: {valid_sizes}"
                }
            changes["size"] = {"old": self.size, "new": size}
            self.size = size
            
        if weight is not None:
            valid_weights = ["light", "normal", "bold", "extra-bold"]
            if weight not in valid_weights:
                return {
                    "success": False,
                    "message": f"Invalid weight. Valid weights: {valid_weights}"
                }
            changes["weight"] = {"old": self.weight, "new": weight}
            self.weight = weight
            
        if color is not None:
            valid_colors = ["default", "primary", "secondary", "success", "warning", "danger", "info"]
            if color not in valid_colors:
                return {
                    "success": False,
                    "message": f"Invalid color. Valid colors: {valid_colors}"
                }
            changes["color"] = {"old": self.color, "new": color}
            self.color = color
            
        if italic is not None:
            changes["italic"] = {"old": self.italic, "new": italic}
            self.italic = italic
            
        if underline is not None:
            changes["underline"] = {"old": self.underline, "new": underline}
            self.underline = underline
            
        if strikethrough is not None:
            changes["strikethrough"] = {"old": self.strikethrough, "new": strikethrough}
            self.strikethrough = strikethrough
        
        return {
            "success": True,
            "changes": changes
        }
    
    def get_computed_style(self) -> Dict[str, Any]:
        """
        Get computed text style properties
        
        Returns:
            Dict containing computed style properties
        """
        return {
            "size": self.size,
            "weight": self.weight,
            "color": self.color,
            "italic": self.italic,
            "underline": self.underline,
            "strikethrough": self.strikethrough
        }
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        props = {
            "content": self.text,
            "element": self.element_type,
            "size": self.size,
            "weight": self.weight,
            "color": self.color,
            "italic": self.italic,
            "underline": self.underline,
            "strikethrough": self.strikethrough,
            **self.get_base_props()
        }
            
        return {
            "type": "text",
            "props": props
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Text':
        """
        Create text component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Text instance
        """
        return Text(
            text=data.get('text', ''),
            element_type=data.get('element', 'span'),
            size=data.get('size', 'medium'),
            weight=data.get('weight', 'normal'),
            color=data.get('color', 'default'),
            italic=data.get('italic', False),
            underline=data.get('underline', False),
            strikethrough=data.get('strikethrough', False)
        )

def create_text(text: str = "", element_type: str = "span", 
                size: str = "medium", weight: str = "normal", 
                color: str = "default", italic: bool = False, 
                underline: bool = False, strikethrough: bool = False, 
                **kwargs) -> Dict[str, Any]:
    """Create a text component"""
    component = Text(text, element_type, size, weight, color, italic, underline, strikethrough, **kwargs)
    return component.to_dict()


"""
Avatar component for representing a user or entity with an image or icon.
"""

from typing import Any, Dict
from ...base import Component


class Avatar(Component):
    """
    Represents a user or entity with an image or icon.
    """
    def __init__(
        self,
        src = None,
        icon = None,
        shape = "circle",
        size = "default",
        **kwargs
    ) -> None:
        self.src = src
        self.icon = icon
        self.shape = shape
        self.size = size
        self.extra_props = kwargs

    def render(self) -> Dict[str, Any]:
        return {
            "type": "avatar",
            "props": {
                "src": self.src,
                "icon": self.icon,
                "shape": self.shape,
                "size": self.size,
                **self.extra_props
            }
        }


"""
Pandas DataFrame Table Plugin for Cacao
Provides table visualization for pandas DataFrames with sorting, pagination, and styling.
"""

from .plugin import PandasTablePlugin

__version__ = "0.1.0"

# Export the plugin class
__all__ = ["PandasTablePlugin"]

# Plugin registration function
def register_plugin():
    """Register the pandas table plugin with Cacao."""
    return PandasTablePlugin()



"""
Image component for displaying images with optional captions and previews.
"""

from typing import Any, Dict
from ...base import Component


class Image(Component):
    """
    Displays images with optional captions and previews.
    """
    def __init__(
        self,
        src,
        alt = "",
        width = None,
        height = None,
        preview = True,
        **kwargs
    ) -> None:
        self.src = src
        self.alt = alt
        self.width = width
        self.height = height
        self.preview = preview
        self.extra_props = kwargs

    def render(self) -> Dict[str, Any]:
        return {
            "type": "image",
            "props": {
                "src": self.src,
                "alt": self.alt,
                "width": self.width,
                "height": self.height,
                "preview": self.preview,
                **self.extra_props
            }
        }


"""
Select Component Python Integration
"""

from typing import Dict, Any, Optional, List
from ...base import Component

class Select(Component):
    """
    Server-side logic for select component
    """
    
    def __init__(self, options: List[Dict[str, Any]], value: Any = None, placeholder: str = None,
                 disabled: bool = False, required: bool = False, **kwargs):
        """
        Initialize select component
        
        Args:
            options: List of option dictionaries with 'value' and 'label' keys
            value: Selected value
            placeholder: Placeholder text
            disabled: Whether select is disabled
            required: Whether select is required
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.options = options
        self.value = value
        self.placeholder = placeholder
        self.disabled = disabled
        self.required = required
    
    def validate(self) -> bool:
        """
        Validate selected value
        
        Returns:
            bool: True if validation passes
        """
        if self.required and self.value is None:
            return False
            
        # Check if selected value exists in options
        if self.value is not None:
            valid_values = [opt.get('value') for opt in self.options]
            return self.value in valid_values
            
        return True
    
    def on_change(self, value: Any) -> Dict[str, Any]:
        """
        Handle select value change
        
        Args:
            value: New selected value
            
        Returns:
            Dict containing response data
        """
        self.value = value
        is_valid = self.validate()
        
        # Find the selected option
        selected_option = None
        for option in self.options:
            if option.get('value') == value:
                selected_option = option
                break
        
        return {
            "success": True,
            "value": self.value,
            "selectedOption": selected_option,
            "valid": is_valid
        }
    
    def add_option(self, value: Any, label: str) -> None:
        """
        Add a new option to the select
        
        Args:
            value: Option value
            label: Option label
        """
        self.options.append({"value": value, "label": label})
    
    def remove_option(self, value: Any) -> bool:
        """
        Remove an option from the select
        
        Args:
            value: Option value to remove
            
        Returns:
            bool: True if option was removed
        """
        for i, option in enumerate(self.options):
            if option.get('value') == value:
                self.options.pop(i)
                # Clear selection if removed option was selected
                if self.value == value:
                    self.value = None
                return True
        return False
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        props = {
            "options": self.options,
            "disabled": self.disabled,
            "required": self.required,
            **self.get_base_props()
        }
        
        if self.value is not None:
            props["value"] = self.value
        if self.placeholder:
            props["placeholder"] = self.placeholder
            
        return {
            "type": "select",
            "props": props
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Select':
        """
        Create select component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Select instance
        """
        return Select(
            options=data.get('options', []),
            value=data.get('value'),
            placeholder=data.get('placeholder'),
            disabled=data.get('disabled', False),
            required=data.get('required', False)
        )

def create_select(options: List[Dict[str, Any]], value: Any = None, placeholder: str = None,
                 disabled: bool = False, required: bool = False, **kwargs) -> Dict[str, Any]:
    """Create a select component"""
    component = Select(options, value, placeholder, disabled, required, **kwargs)
    return component.to_dict()


import pytest
from cacao.ui.components.base import Component
from cacao.ui.components.layout import Grid, Column
from cacao.ui.components.data import (
    Table, Plot, Table, List, Card, Image, Badge, Avatar, Tag, Timeline
)
from cacao.ui.components.inputs import Slider, Input, TextArea, SearchInput, Select, Checkbox, Radio, Switch, DatePicker, TimePicker
from cacao.ui.components.forms import Form, FormItem
from cacao.ui.components.navigation import Menu, Tabs, Dropdown, Pagination, Steps
from typing import Dict, Any

class TestButton(Component):
    __test__ = False  # Prevent pytest from collecting this class
    def __init__(self, label="Test Button"):
        super().__init__()
        self.label = label
    
    def render(self) -> Dict[str, Any]:
        return {"type": "button", "props": {"label": self.label}}

class TestText(Component):
    __test__ = False  # Prevent pytest from collecting this class
    def __init__(self, content="Test Content"):
        super().__init__()
        self.content = content
    
    def render(self) -> Dict[str, Any]:
        return {"type": "text", "props": {"content": self.content}}

def test_grid_component():
    button = TestButton("Click me")
    
    grid = Grid(children=[button], columns=2)
    rendered = grid.render()
    
    assert rendered["type"] == "grid"
    assert rendered["props"]["columns"] == 2
    assert len(rendered["props"]["children"]) == 1
    assert rendered["props"]["children"][0]["type"] == "button"

def test_column_component():
    text1 = TestText("First item")
    text2 = TestText("Second item")
    
    column = Column(children=[text1, text2])
    rendered = column.render()
    
    assert rendered["type"] == "column"
    assert len(rendered["props"]["children"]) == 2
    assert rendered["props"]["children"][0]["props"]["content"] == "First item"
    assert rendered["props"]["children"][1]["props"]["content"] == "Second item"

def test_table_component():
    table = Table(
        headers=["Name", "Age", "Location"],
        rows=[
            ["Alice", "30", "New York"],
            ["Bob", "25", "San Francisco"]
        ]
    )
    
    rendered = table.render()
    assert rendered["type"] == "table"
    assert "headers" in rendered["props"]
    assert "rows" in rendered["props"]
    assert len(rendered["props"]["headers"]) == 3
    assert len(rendered["props"]["rows"]) == 2

def test_plot_component():
    plot_data = {
        "x": [1, 2, 3, 4, 5],
        "y": [10, 15, 13, 17, 20]
    }
    
    plot = Plot(data=plot_data, title="Sample Plot")
    
    rendered = plot.render()
    assert rendered["type"] == "plot"
    assert rendered["props"]["title"] == "Sample Plot"
    assert "data" in rendered["props"]
    assert "x" in rendered["props"]["data"]
    assert "y" in rendered["props"]["data"]

def test_slider_component():
    slider = Slider(min_value=0, max_value=100, step=1, value=50)
    rendered = slider.render()
    assert rendered["type"] == "slider"
    assert rendered["props"]["min"] == 0
    assert rendered["props"]["max"] == 100
    assert rendered["props"]["step"] == 1
    assert rendered["props"]["value"] == 50

def test_form_component():
    form = Form(fields={"username": {"type": "text", "placeholder": "Enter your name"}})
    rendered = form.render()
    assert rendered["type"] == "form"
    assert "fields" in rendered["props"]
    assert "username" in rendered["props"]["fields"]
    assert rendered["props"]["fields"]["username"]["type"] == "text"

def test_enhanced_table_component():
    columns = [
        {"title": "Name", "dataIndex": "name", "key": "name"},
        {"title": "Age", "dataIndex": "age", "key": "age"},
        {"title": "Address", "dataIndex": "address", "key": "address"}
    ]
    data_source = [
        {"key": "1", "name": "John Brown", "age": 32, "address": "New York No. 1 Lake Park"},
        {"key": "2", "name": "Jim Green", "age": 42, "address": "London No. 1 Lake Park"}
    ]
    pagination = {"page_size": 10, "current": 1}

    table = Table(columns=columns, data_source=data_source, pagination=pagination)
    rendered = table.render()
    
    assert rendered["type"] == "table"
    assert rendered["props"]["columns"] == columns
    assert rendered["props"]["dataSource"] == data_source
    assert rendered["props"]["pagination"] == pagination

def test_list_component():
    items = [
        {"title": "Item 1", "description": "Description 1"},
        {"title": "Item 2", "description": "Description 2"}
    ]
    
    list_component = List(items=items, bordered=True)
    rendered = list_component.render()
    
    assert rendered["type"] == "list"
    assert rendered["props"]["items"] == items
    assert rendered["props"]["bordered"] == True

def test_card_component():
    card = Card(
        children="Card content",
        title="Card Title",
        bordered=True
    )
    rendered = card.render()
    
    assert rendered["type"] == "card"
    assert rendered["props"]["children"] == "Card content"
    assert rendered["props"]["title"] == "Card Title"
    assert rendered["props"]["bordered"] == True

def test_image_component():
    image = Image(
        src="https://example.com/image.jpg",
        alt="Example Image",
        width=200,
        height=150
    )
    rendered = image.render()
    
    assert rendered["type"] == "image"
    assert rendered["props"]["src"] == "https://example.com/image.jpg"
    assert rendered["props"]["alt"] == "Example Image"
    assert rendered["props"]["width"] == 200
    assert rendered["props"]["height"] == 150

def test_badge_component():
    badge = Badge(
        count=5,
        children=TestText("Notifications"),
        dot=False
    )
    rendered = badge.render()
    
    assert rendered["type"] == "badge"
    assert rendered["props"]["count"] == 5
    assert rendered["props"]["children"]["type"] == "text"
    assert rendered["props"]["dot"] == False

def test_avatar_component():
    avatar = Avatar(
        src="https://example.com/avatar.jpg",
        shape="circle",
        size="large"
    )
    rendered = avatar.render()
    
    assert rendered["type"] == "avatar"
    assert rendered["props"]["src"] == "https://example.com/avatar.jpg"
    assert rendered["props"]["shape"] == "circle"
    assert rendered["props"]["size"] == "large"

def test_tag_component():
    tag = Tag(
        content="New",
        color="blue",
        closable=True
    )
    rendered = tag.render()
    
    assert rendered["type"] == "tag"
    assert rendered["props"]["content"] == "New"
    assert rendered["props"]["color"] == "blue"
    assert rendered["props"]["closable"] == True

def test_timeline_component():
    items = [
        {"label": "2015", "content": "Created"},
        {"label": "2020", "content": "Updated"},
        {"label": "2025", "content": "Completed"}
    ]
    
    timeline = Timeline(
        items=items,
        mode="left",
        reverse=False
    )
    rendered = timeline.render()
    
    assert rendered["type"] == "timeline"
    assert rendered["props"]["items"] == items
    assert rendered["props"]["mode"] == "left"
    assert rendered["props"]["reverse"] == False

def test_input_component():
    input_field = Input(
        input_type="text",
        value="test value",
        placeholder="Enter text",
        disabled=False
    )
    rendered = input_field.render()
    assert rendered["type"] == "input"
    assert rendered["props"]["inputType"] == "text"
    assert rendered["props"]["value"] == "test value"
    assert rendered["props"]["placeholder"] == "Enter text"
    assert rendered["props"]["disabled"] == False

def test_textarea_component():
    textarea = TextArea(
        value="sample text",
        placeholder="Enter description",
        rows=5
    )
    rendered = textarea.render()
    assert rendered["type"] == "textarea"
    assert rendered["props"]["value"] == "sample text"
    assert rendered["props"]["placeholder"] == "Enter description"
    assert rendered["props"]["rows"] == 5

def test_search_input_component():
    search = SearchInput(
        value="search term",
        placeholder="Search..."
    )
    rendered = search.render()
    assert rendered["type"] == "search"
    assert rendered["props"]["value"] == "search term"
    assert rendered["props"]["placeholder"] == "Search..."

def test_select_component():
    options = [
        {"label": "Option 1", "value": 1},
        {"label": "Option 2", "value": 2}
    ]
    select = Select(
        options=options,
        value=1,
        placeholder="Select an option"
    )
    rendered = select.render()
    assert rendered["type"] == "select"
    assert rendered["props"]["options"] == options
    assert rendered["props"]["value"] == 1
    assert rendered["props"]["placeholder"] == "Select an option"

def test_checkbox_component():
    checkbox = Checkbox(
        label="Accept terms",
        checked=True
    )
    rendered = checkbox.render()
    assert rendered["type"] == "checkbox"
    assert rendered["props"]["label"] == "Accept terms"
    assert rendered["props"]["checked"] == True

def test_radio_component():
    options = [
        {"label": "Option A", "value": "a"},
        {"label": "Option B", "value": "b"}
    ]
    radio = Radio(
        options=options,
        value="a"
    )
    rendered = radio.render()
    assert rendered["type"] == "radio"
    assert rendered["props"]["options"] == options
    assert rendered["props"]["value"] == "a"

def test_switch_component():
    switch = Switch(checked=True)
    rendered = switch.render()
    assert rendered["type"] == "switch"
    assert rendered["props"]["checked"] == True

def test_datepicker_component():
    datepicker = DatePicker(
        value="2025-04-13",
        range=False
    )
    rendered = datepicker.render()
    assert rendered["type"] == "datepicker"
    assert rendered["props"]["value"] == "2025-04-13"
    assert rendered["props"]["range"] == False

def test_timepicker_component():
    timepicker = TimePicker(value="14:30")
    rendered = timepicker.render()
    assert rendered["type"] == "timepicker"
    assert rendered["props"]["value"] == "14:30"

def test_enhanced_form_component():
    fields = [
        {"name": "username", "label": "Username", "type": "text"},
        {"name": "password", "label": "Password", "type": "password"}
    ]
    validation_rules = {
        "username": [{"required": True, "message": "Username is required"}]
    }
    form = Form(
        fields=fields,
        layout="vertical",
        validation_rules=validation_rules
    )
    rendered = form.render()
    assert rendered["type"] == "form"
    assert rendered["props"]["fields"] == fields
    assert rendered["props"]["layout"] == "vertical"
    assert rendered["props"]["validationRules"] == validation_rules

def test_menu_component():
    items = [
        {"key": "home", "label": "Home"},
        {"key": "about", "label": "About"}
    ]
    menu = Menu(
        items=items,
        mode="horizontal",
        selected_keys=["home"]
    )
    rendered = menu.render()
    assert rendered["type"] == "menu"
    assert rendered["props"]["items"] == items
    assert rendered["props"]["mode"] == "horizontal"
    assert rendered["props"]["selectedKeys"] == ["home"]

def test_tabs_component():
    items = [
        {"key": "tab1", "label": "Tab 1", "content": "Content 1"},
        {"key": "tab2", "label": "Tab 2", "content": "Content 2"}
    ]
    tabs = Tabs(
        items=items,
        active_key="tab1"
    )
    rendered = tabs.render()
    assert rendered["type"] == "tabs"
    assert rendered["props"]["items"] == items
    assert rendered["props"]["activeKey"] == "tab1"

def test_dropdown_component():
    items = [
        {"key": "item1", "label": "Item 1"},
        {"key": "item2", "label": "Item 2"}
    ]
    dropdown = Dropdown(
        items=items,
        trigger="click",
        placement="bottomLeft"
    )
    rendered = dropdown.render()
    assert rendered["type"] == "dropdown"
    assert rendered["props"]["items"] == items
    assert rendered["props"]["trigger"] == "click"
    assert rendered["props"]["placement"] == "bottomLeft"

def test_pagination_component():
    pagination = Pagination(
        total=100,
        current=1,
        page_size=10
    )
    rendered = pagination.render()
    assert rendered["type"] == "pagination"
    assert rendered["props"]["total"] == 100
    assert rendered["props"]["current"] == 1
    assert rendered["props"]["pageSize"] == 10


"""
Checkbox Component Python Integration
Example of how to create Python logic for form components
"""

from typing import Dict, Any, Optional
from cacao.ui.components.base import Component

class Checkbox(Component):
    """
    Server-side logic for checkbox component
    """
    
    def __init__(self, checked: bool = False, label: str = "", disabled: bool = False, **kwargs):
        """
        Initialize checkbox component
        
        Args:
            checked: Whether checkbox is initially checked
            label: Label text for the checkbox
            disabled: Whether checkbox is disabled
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.checked = checked
        self.label = label
        self.disabled = disabled
    
    def validate(self) -> bool:
        """
        Validate checkbox state
        
        Returns:
            bool: True if validation passes
        """
        # Add any validation logic here
        return True
    
    def on_change(self, checked: bool) -> Dict[str, Any]:
        """
        Handle checkbox state change
        
        Args:
            checked: New checked state
            
        Returns:
            Dict containing response data
        """
        self.checked = checked
        
        # Add any business logic here
        # For example, updating related data, triggering events, etc.
        
        return {
            "success": True,
            "checked": self.checked,
            "message": f"Checkbox {'checked' if checked else 'unchecked'}"
        }
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        return {
            "type": "checkbox",
            "props": {
                "checked": self.checked,
                "label": self.label,
                "disabled": self.disabled,
                "className": getattr(self, 'className', 'checkbox'),
                **self.get_base_props()
            }
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Checkbox':
        """
        Create checkbox component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Checkbox instance
        """
        return Checkbox(
            checked=data.get('checked', False),
            label=data.get('label', ''),
            disabled=data.get('disabled', False)
        )

# Example usage:
def create_checkbox_example():
    """Example of how to use the checkbox component"""
    checkbox = Checkbox(
        checked=False,
        label="Accept Terms and Conditions",
        disabled=False
    )
    
    # Render component
    return checkbox.to_dict()

def handle_checkbox_event(component_data: Dict[str, Any], event_data: Dict[str, Any]):
    """
    Example event handler for checkbox component
    
    Args:
        component_data: Current component state
        event_data: Event data from frontend
    """
    checkbox = Checkbox.from_form_data(component_data.get('props', {}))
    
    # Handle the change event
    new_checked = event_data.get('checked', False)
    result = checkbox.on_change(new_checked)
    
    return {
        "component": checkbox.to_dict(),
        "result": result
    }


"""
Timeline component for displaying a chronological sequence of events.
"""

from typing import Any, Dict
from ...base import Component


class Timeline(Component):
    """
    Displays a chronological sequence of events.
    """
    def __init__(
        self,
        items,
        mode = "left",
        reverse = False,
        **kwargs
    ) -> None:
        self.items = items
        self.mode = mode
        self.reverse = reverse
        self.extra_props = kwargs

    def render(self) -> Dict[str, Any]:
        return {
            "type": "timeline",
            "props": {
                "items": self.items,
                "mode": self.mode,
                "reverse": self.reverse,
                **self.extra_props
            }
        }


"""
Search Component Python Integration
"""

from typing import Dict, Any, Optional
from ...base import Component

class SearchInput(Component):
    """
    Server-side logic for search component
    """
    
    def __init__(self, value: str = "", placeholder: str = None, disabled: bool = False, **kwargs):
        """
        Initialize search component
        
        Args:
            value: Initial search value
            placeholder: Placeholder text
            disabled: Whether search is disabled
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.value = value
        self.placeholder = placeholder
        self.disabled = disabled
    
    def validate(self) -> bool:
        """
        Validate search value
        
        Returns:
            bool: True if validation passes
        """
        # Basic validation - search can be empty
        return True
    
    def on_search(self, value: str) -> Dict[str, Any]:
        """
        Handle search query
        
        Args:
            value: Search query value
            
        Returns:
            Dict containing search results or response data
        """
        self.value = value
        
        # Add search logic here
        # For example, filter data, call search API, etc.
        
        return {
            "success": True,
            "query": self.value,
            "results": [],  # Replace with actual search results
            "total": 0
        }
    
    def clear_search(self) -> Dict[str, Any]:
        """
        Clear search value
        
        Returns:
            Dict containing response data
        """
        self.value = ""
        
        return {
            "success": True,
            "query": "",
            "cleared": True
        }
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        props = {
            "value": self.value,
            "disabled": self.disabled,
            **self.get_base_props()
        }
        
        if self.placeholder:
            props["placeholder"] = self.placeholder
            
        return {
            "type": "search",
            "props": props
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'SearchInput':
        """
        Create search component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            SearchInput instance
        """
        return SearchInput(
            value=data.get('value', ''),
            placeholder=data.get('placeholder'),
            disabled=data.get('disabled', False)
        )

def create_search(value: str = "", placeholder: str = None, disabled: bool = False, **kwargs) -> Dict[str, Any]:
    """Create a search component"""
    component = SearchInput(value, placeholder, disabled, **kwargs)
    return component.to_dict()


"""
Example of using the icon registry system in Cacao with a sidebar layout.

This demonstrates how to use custom icons and FontAwesome icons in separate tabs
using Cacao's SidebarLayout component.
"""

import cacao
from cacao import Component
from cacao.ui.components.sidebar_layout import SidebarLayout
from typing import Dict, Any, Optional

app = cacao.App()

# Register a custom user icon
custom_svg = """
<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24" height="24">
  <path d="M12,4C14.21,4 16,5.79 16,8C16,10.21 14.21,12 12,12C9.79,12 8,10.21 8,8C8,5.79 9.79,4 12,4M12,14C16.42,14 20,15.79 20,18V20H4V18C4,15.79 7.58,14 12,14Z" fill="currentColor"/>
</svg>
"""
cacao.icon_registry.register_icon("user", custom_svg)

class CustomIconsPage(Component):
    def _create_icon_card(self, label: str, icon_markup: str) -> Dict[str, Any]:
        return {
            "type": "div",
            "props": {
                "style": {
                    "backgroundColor": "#F5F5F5",
                    "borderRadius": "8px",
                    "padding": "20px",
                    "boxShadow": "0 2px 4px rgba(0,0,0,0.1)",
                    "display": "flex",
                    "alignItems": "center",
                    "gap": "15px"
                },
                "children": [
                    {
                        "type": "div",
                        "props": {
                            "style": {
                                "width": "60px",
                                "height": "60px",
                                "display": "flex",
                                "alignItems": "center",
                                "justifyContent": "center",
                                "backgroundColor": "white",
                                "borderRadius": "12px",
                                "boxShadow": "0 2px 4px rgba(0,0,0,0.05)"
                            },
                            "children": [
                                {
                                    "type": "text",
                                    "props": {
                                        "content": icon_markup
                                    }
                                }
                            ]
                        }
                    },
                    {
                        "type": "text",
                        "props": {
                            "content": label,
                            "style": {
                                "fontSize": "16px",
                                "color": "#2D2013"
                            }
                        }
                    }
                ]
            }
        }

    def render(self, ui_state: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """Render the custom icons page.
        
        Args:
            ui_state: Optional UI state data
            
        Returns:
            Dict[str, Any]: Component UI definition
        """
        # Define the custom icons to display with their names, icons, colors, and sizes
        custom_icons = [
            ("Home Icon", "home", "#6B4226", 32),
            ("Bean Icon", "bean", "#D4A76A", 32),
            ("User Icon", "user", "#3498db", 32),
            ("Settings Icon", "settings", "#9b59b6", 32),
            ("Cloud Icon", "cloud", "#3498db", 32),
            ("Fire Icon", "fire", "#e74c3c", 32),
            ("Chef Hat", "chef-hat", "#2c3e50", 32),
            ("Star Icon", "star", "#f1c40f", 36),
            ("Trash Icon", "trash", "#e74c3c", 32),
            ("Whisk Icon", "whisk", "#7f8c8d", 36),
            ("Menu Icon", "menu", "#2c3e50", 32),
            ("Hot Drink", "hot-drink", "#6f4e37", 32)
        ]
        
        return {
            "type": "div",
            "props": {
                "style": {
                    "padding": "20px"
                },
                "children": [
                    {
                        "type": "h2",
                        "props": {
                            "content": "Custom Icons",
                            "style": {
                                "color": "#6B4226",
                                "marginBottom": "20px"
                            }
                        }
                    },
                    {
                        "type": "div",
                        "props": {
                            "style": {
                                "display": "grid",
                                "gridTemplateColumns": "repeat(auto-fill, minmax(200px, 1fr))",
                                "gap": "20px"
                            },
                            "children": [
                                self._create_icon_card(
                                    label, 
                                    f"{{%icon-ca-{icon} size={size} color={color}%}}"
                                )
                                for label, icon, color, size in custom_icons
                            ]
                        }
                    }
                ]
            }
        }

class FontAwesomePage(Component):
    """Component for displaying FontAwesome icons with proper naming conventions."""
    
    def _create_fa_icon_card(self, icon_name: str, color: str = "#6B4226", size: int = 32) -> Dict[str, Any]:
        """Create a card displaying a FontAwesome icon with its name.
        
        Args:
            icon_name: The FontAwesome icon name without the fa- prefix
            color: The color for the icon in hex format
            size: The size of the icon in pixels
            
        Returns:
            Dict[str, Any]: The icon card component definition
        """
        return {
            "type": "div",
            "props": {
                "style": {
                    "backgroundColor": "#F5F5F5",
                    "borderRadius": "8px",
                    "padding": "20px",
                    "boxShadow": "0 2px 4px rgba(0,0,0,0.1)",
                    "display": "flex",
                    "alignItems": "center",
                    "gap": "15px"
                },
                "children": [
                    {
                        "type": "div",
                        "props": {
                            "style": {
                                "width": "60px",
                                "height": "60px",
                                "display": "flex",
                                "alignItems": "center",
                                "justifyContent": "center",
                                "backgroundColor": "white",
                                "borderRadius": "12px",
                                "boxShadow": "0 2px 4px rgba(0,0,0,0.05)"
                            },
                            "content": f"{{%icon-fa-{icon_name} size={size} color={color}%}}"
                        }
                    },
                    {
                        "type": "div",
                        "props": {
                            "content": f"fa-{icon_name}",
                            "style": {
                                "fontSize": "16px",
                                "color": "#2D2013"
                            }
                        }
                    }
                ]
            }
        }

    def render(self, ui_state: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """Render the FontAwesome icons page.
        
        Args:
            ui_state: Optional UI state data
            
        Returns:
            Dict[str, Any]: Component UI definition
        """
        # Define the FontAwesome icons to display with their colors
        fa_icons = [
            ("star", "#FFD700"),       # Gold
            ("heart", "#ff5555"),      # Red
            ("check", "#4CAF50"),      # Green
            ("user", "#3498db"),       # Blue
            ("cog", "#9b59b6"),        # Purple
            ("bell", "#f39c12"),       # Orange
            ("file", "#2ecc71"),       # Green
            ("calendar", "#e74c3c"),    # Red
            ("home", "#1abc9c"),       # Teal
            ("search", "#34495e"),     # Dark Blue
            ("envelope", "#7f8c8d"),    # Gray
            ("lock", "#f1c40f"),        # Yellow
            ("trash", "#e74c3c"),       # Red
            ("coffee", "#6f4e37"),      # Brown
            ("download", "#2980b9"),    # Blue
            ("upload", "#2980b9"),      # Blue
            ("camera", "#9b59b6"),      # Purple
            ("pencil", "#f39c12"),      # Orange
            ("globe", "#3498db"),       # Blue
            ("microphone", "#e67e22"),  # Orange
            ("music", "#2ecc71"),       # Green
            ("shopping-cart", "#e74c3c"), # Red
            ("plus", "#2c3e50"),        # Dark Blue/Gray
            ("minus", "#2c3e50")        # Dark Blue/Gray
        ]

        
        return {
            "type": "div",
            "props": {
                "style": {
                    "padding": "20px"
                },
                "children": [
                    {
                        "type": "h2",
                        "props": {
                            "content": "FontAwesome Icons",
                            "style": {
                                "color": "#6B4226",
                                "marginBottom": "20px"
                            }
                        }
                    },
                    {
                        "type": "div",
                        "props": {
                            "style": {
                                "display": "grid",
                                "gridTemplateColumns": "repeat(auto-fill, minmax(250px, 1fr))",
                                "gap": "20px"
                            },
                            "children": [
                                self._create_fa_icon_card(icon_name, color) 
                                for icon_name, color in fa_icons
                            ]
                        }
                    }
                ]
            }
        }

class GuidelinePage(Component):
    def render(self, ui_state: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        return {
            "type": "div",
            "props": {
                "style": {
                    "padding": "20px"
                },
                "children": [
                    {
                        "type": "h2",
                        "props": {
                            "content": "Icon Usage Guidelines",
                            "style": {
                                "color": "#6B4226",
                                "marginBottom": "20px"
                            }
                        }
                    },
                    {
                        "type": "pre",
                        "props": {
                            "style": {
                                "backgroundColor": "#f5f5f5",
                                "padding": "20px",
                                "borderRadius": "8px",
                                "overflow": "auto",
                                "fontSize": "14px",
                                "lineHeight": "1.5"
                            },
                            "content": """# Custom SVG icons
{%icon-ca-name size=32%}
{%icon-ca-name size=32 color=#ff0000%}

# FontAwesome icons
{%icon-fa-name size=32%}
{%icon-fa-name size=32 color=#00ff00%}"""
                        }
                    }
                ]
            }
        }

# Define navigation items with icons
nav_items = [
    {"id": "custom", "label": "Custom Icons", "icon": "{%icon-ca-home size=16 color=#fff%}"},
    {"id": "fontawesome", "label": "FontAwesome", "icon": "{%icon-fa-star size=16 color=#fff%}"},
    {"id": "guidelines", "label": "Guidelines", "icon": "{%icon-fa-book size=16 color=#fff%}"}
]

# Create page instances
custom_icons_page = CustomIconsPage()
fontawesome_page = FontAwesomePage()
guidelines_page = GuidelinePage()

# Define content components for each page
content_components = {
    "custom": custom_icons_page,
    "fontawesome": fontawesome_page,
    "guidelines": guidelines_page
}

# Create the sidebar layout
sidebar_layout = SidebarLayout(
    nav_items=nav_items,
    content_components=content_components,
    app_title="Cacao Icons"
)

@app.mix("/")
def home() -> Dict[str, Any]:
    """Main route handler for the icon demo app."""
    return sidebar_layout.render()

if __name__ == "__main__":
    import argparse
    
    parser = argparse.ArgumentParser(description="Icon Registry Example")
    parser.add_argument("--mode", choices=["web", "desktop"], default="web",
                       help="Run mode: 'web' for browser or 'desktop' for PWA window")
    parser.add_argument("--width", type=int, default=1024, help="Window width (desktop mode only)")
    parser.add_argument("--height", type=int, default=768, help="Window height (desktop mode only)")
    
    args = parser.parse_args()
    
    print("Starting icon demo app...")
    app.brew(
        type=args.mode,
        title="Cacao Icons",
        width=args.width,
        height=args.height,
        resizable=True,
        fullscreen=False
    )


import pytest
from cacao.extensions.plugins import Plugin, register_plugin, PLUGINS

class TestPlugin(Plugin):
    """Test plugin implementation for testing purposes."""
    __test__ = False  # Prevent pytest from collecting this class
    
    def __init__(self, name: str):
        super().__init__(name)
        self.registered = False
        self.data = {}
    
    def register(self) -> None:
        """Implementation of the required register method."""
        self.registered = True

@pytest.fixture
def reset_plugins():
    """Fixture to reset the global PLUGINS list before and after tests."""
    original_plugins = PLUGINS.copy()
    PLUGINS.clear()
    yield
    PLUGINS.clear()
    PLUGINS.extend(original_plugins)

def test_plugin_registration(reset_plugins):
    plugin = TestPlugin("test-plugin")
    register_plugin(plugin)
    assert plugin in PLUGINS
    assert plugin.registered is True
    assert len(PLUGINS) == 1

def test_multiple_plugin_registration(reset_plugins):
    plugin1 = TestPlugin("plugin-1")
    plugin2 = TestPlugin("plugin-2")
    register_plugin(plugin1)
    register_plugin(plugin2)
    assert len(PLUGINS) == 2
    assert plugin1 in PLUGINS
    assert plugin2 in PLUGINS

def test_plugin_custom_functionality(reset_plugins):
    plugin = TestPlugin("data-plugin")
    register_plugin(plugin)
    plugin.data["theme"] = "dark"
    plugin.data["color"] = "#333"
    assert plugin.data["theme"] == "dark"
    assert plugin.data["color"] == "#333"

def test_abstract_plugin_registration():
    abstract_plugin = Plugin("abstract")
    with pytest.raises(NotImplementedError):
        abstract_plugin.register()


"""
Plot component for displaying charts and graphs.
"""

from typing import Any, Dict
from ...base import Component


class Plot(Component):
    """
    A plot component for displaying charts and graphs.
    """
    def __init__(self, data, title = "") -> None:
        self.data = data
        self.title = title

    def render(self) -> Dict[str, Any]:
        return {
            "type": "plot",
            "props": {
                "data": self.data,
                "title": self.title
            }
        }


"""
UI Components package for Cacao framework.
Provides base components and common UI elements.
"""

from .base import Component, ComponentProps

from .inputs import (
    InputGroup,  Cascader
)

from .forms import (
    Slider, Input, Textarea, SearchInput, 
    Select, Checkbox, Radio, Switch, Rate, 
    Datepicker, Timepicker, Upload
)

from .data import (
    Plot, List, Descriptions, Tooltip, TreeViewer, Popover,
    Card, Carousel, Collapse, Image, Badge, Avatar, Tag, Timeline
)

from .layout import Grid, Column
from .sidebar_layout import SidebarLayout
from .form_components import Form, FormItem
from .navigation import Menu, NavItem, Navbar, Tabs, Breadcrumb
from .ui import Button, Sidebar, Text

__all__ = [
    # Base
    "Component",
    "ComponentProps",
    
    # Inputs
    "Slider",
    "Input",
    "Textarea",
    "SearchInput",
    "InputGroup",
    "Select",
    "Cascader",
    "Checkbox",
    "Radio",
    "Switch",
    "Rate",
    "Datepicker",
    "Timepicker",
    "Upload",
    
    # Data
    "Plot",
    "List",
    "Descriptions",
    "Tooltip",
    "TreeViewer",
    "Popover",
    "Card",
    "Carousel",
    "Collapse",
    "Image",
    "Badge",
    "Avatar",
    "Tag",
    "Timeline",
    
    # Layout
    "Grid",
    "Column",
    "SidebarLayout",
    
    # Forms
    "Form",
    "FormItem",
    
    # Navigation
    "Menu",
    "NavItem",
    "Navbar",
    "Tabs",
    "Breadcrumb",
    
    # UI Components
    "Button",
    "Sidebar",
    "Text"
]


"""
List component for presenting data in a vertical sequence.
"""

from typing import Any, Dict
from ...base import Component


class List(Component):
    """
    Presents data in a vertical sequence (simple, bordered, with actions).
    """
    def __init__(
        self,
        items,
        bordered = False,
        size = "default",
        **kwargs
    ) -> None:
        self.items = items
        self.bordered = bordered
        self.size = size
        self.extra_props = kwargs

    def render(self) -> Dict[str, Any]:
        return {
            "type": "list",
            "props": {
                "items": self.items,
                "bordered": self.bordered,
                "size": self.size,
                **self.extra_props
            }
        }


"""
Card component for displaying related information in a concise format.
"""

from typing import Any, Dict
from ...base import Component


class Card(Component):
    """
    A container for displaying related information in a concise format.
    """
    def __init__(
        self,
        children,
        title = "",
        bordered = True,
        extra = None,
        **kwargs
    ) -> None:
        self.children = children
        self.title = title
        self.bordered = bordered
        self.extra = extra
        self.extra_props = kwargs

    def render(self) -> Dict[str, Any]:
        return {
            "type": "card",
            "props": {
                "children": self.children,
                "title": self.title,
                "bordered": self.bordered,
                "extra": self.extra,
                **self.extra_props
            }
        }


"""
Input Component Python Integration
"""

from typing import Dict, Any, Optional
from ...base import Component

class Input(Component):
    """
    Server-side logic for input component
    """
    
    def __init__(self, input_type: str = "text", value: str = "", placeholder: str = None, 
                 disabled: bool = False, required: bool = False, **kwargs):
        """
        Initialize input component
        
        Args:
            input_type: Type of input (text, email, password, etc.)
            value: Initial value
            placeholder: Placeholder text
            disabled: Whether input is disabled
            required: Whether input is required
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.input_type = input_type
        self.value = value
        self.placeholder = placeholder
        self.disabled = disabled
        self.required = required
    
    def validate(self) -> bool:
        """
        Validate input value
        
        Returns:
            bool: True if validation passes
        """
        if self.required and not self.value:
            return False
            
        # Add specific validation based on input type
        if self.input_type == "email":
            import re
            email_pattern = r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$'
            return re.match(email_pattern, self.value) is not None
            
        return True
    
    def on_change(self, value: str) -> Dict[str, Any]:
        """
        Handle input value change
        
        Args:
            value: New input value
            
        Returns:
            Dict containing response data
        """
        self.value = value
        is_valid = self.validate()
        
        return {
            "success": True,
            "value": self.value,
            "valid": is_valid
        }
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        props = {
            "inputType": self.input_type,
            "value": self.value,
            "disabled": self.disabled,
            "required": self.required,
            **self.get_base_props()
        }
        
        if self.placeholder:
            props["placeholder"] = self.placeholder
            
        return {
            "type": "input",
            "props": props
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Input':
        """
        Create input component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Input instance
        """
        return Input(
            input_type=data.get('inputType', 'text'),
            value=data.get('value', ''),
            placeholder=data.get('placeholder'),
            disabled=data.get('disabled', False),
            required=data.get('required', False)
        )

def create_input(input_type: str = "text", value: str = "", placeholder: str = None,
                disabled: bool = False, required: bool = False, **kwargs) -> Dict[str, Any]:
    """Create an input component"""
    component = Input(input_type, value, placeholder, disabled, required, **kwargs)
    return component.to_dict()


"""
Tag component for labeling content with keywords or categories.
"""

from typing import Any, Dict
from ...base import Component


class Tag(Component):
    """
    Labels content with keywords or categories.
    """
    def __init__(
        self,
        content,
        color = None,
        closable = False,
        **kwargs
    ) -> None:
        self.content = content
        self.color = color
        self.closable = closable
        self.extra_props = kwargs

    def render(self) -> Dict[str, Any]:
        return {
            "type": "tag",
            "props": {
                "content": self.content,
                "color": self.color,
                "closable": self.closable,
                **self.extra_props
            }
        }


"""
Datepicker Component Python Integration
"""

from typing import Dict, Any, Optional
from datetime import datetime, date
from ...base import Component

class Datepicker(Component):
    """
    Server-side logic for datepicker component
    """
    
    def __init__(self, value: str = None, disabled: bool = False, min_date: str = None,
                 max_date: str = None, **kwargs):
        """
        Initialize datepicker component
        
        Args:
            value: Initial date value (YYYY-MM-DD format)
            disabled: Whether datepicker is disabled
            min_date: Minimum selectable date
            max_date: Maximum selectable date
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.value = value
        self.disabled = disabled
        self.min_date = min_date
        self.max_date = max_date
    
    def validate(self) -> bool:
        """
        Validate date value
        
        Returns:
            bool: True if validation passes
        """
        if not self.value:
            return True
            
        try:
            # Parse the date to ensure it's valid
            parsed_date = datetime.strptime(self.value, '%Y-%m-%d').date()
            
            # Check min/max constraints
            if self.min_date:
                min_parsed = datetime.strptime(self.min_date, '%Y-%m-%d').date()
                if parsed_date < min_parsed:
                    return False
                    
            if self.max_date:
                max_parsed = datetime.strptime(self.max_date, '%Y-%m-%d').date()
                if parsed_date > max_parsed:
                    return False
                    
            return True
        except ValueError:
            return False
    
    def on_change(self, value: str) -> Dict[str, Any]:
        """
        Handle date value change
        
        Args:
            value: New date value
            
        Returns:
            Dict containing response data
        """
        old_value = self.value
        self.value = value
        is_valid = self.validate()
        
        # Parse date for additional info
        parsed_date = None
        if value and is_valid:
            try:
                parsed_date = datetime.strptime(value, '%Y-%m-%d').date()
            except ValueError:
                pass
        
        return {
            "success": True,
            "value": self.value,
            "previousValue": old_value,
            "valid": is_valid,
            "parsedDate": {
                "year": parsed_date.year if parsed_date else None,
                "month": parsed_date.month if parsed_date else None,
                "day": parsed_date.day if parsed_date else None,
                "weekday": parsed_date.weekday() if parsed_date else None
            } if parsed_date else None
        }
    
    def set_to_today(self) -> Dict[str, Any]:
        """
        Set date to today
        
        Returns:
            Dict containing response data
        """
        today = date.today().strftime('%Y-%m-%d')
        return self.on_change(today)
    
    def clear_date(self) -> Dict[str, Any]:
        """
        Clear date value
        
        Returns:
            Dict containing response data
        """
        return self.on_change("")
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        props = {
            "disabled": self.disabled,
            **self.get_base_props()
        }
        
        if self.value:
            props["value"] = self.value
        if self.min_date:
            props["min"] = self.min_date
        if self.max_date:
            props["max"] = self.max_date
            
        return {
            "type": "datepicker",
            "props": props
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Datepicker':
        """
        Create datepicker component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Datepicker instance
        """
        return Datepicker(
            value=data.get('value'),
            disabled=data.get('disabled', False),
            min_date=data.get('min'),
            max_date=data.get('max')
        )

def create_datepicker(value: str = None, disabled: bool = False, min_date: str = None,
                     max_date: str = None, **kwargs) -> Dict[str, Any]:
    """Create a datepicker component"""
    component = Datepicker(value, disabled, min_date, max_date, **kwargs)
    return component.to_dict()


"""
Rate Component Python Integration
"""

from typing import Dict, Any, Optional
from ...base import Component

class Rate(Component):
    """
    Server-side logic for rate component
    """
    
    def __init__(self, max_rating: int = 5, value: float = 0, allow_half: bool = True,
                 onChange: Dict[str, Any] = None, **kwargs):
        """
        Initialize rate component
        
        Args:
            max_rating: Maximum rating value
            value: Current rating value
            allow_half: Whether to allow half-star ratings
            onChange: Change handler configuration
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.max_rating = max_rating
        self.value = value
        self.allow_half = allow_half
        self.onChange = onChange
    
    def validate(self) -> bool:
        """
        Validate rating value
        
        Returns:
            bool: True if validation passes
        """
        # Check if value is within bounds
        if self.value < 0 or self.value > self.max_rating:
            return False
            
        # Check half-star constraints
        if not self.allow_half and self.value % 1 != 0:
            return False
            
        return True
    
    def on_rate(self, value: float) -> Dict[str, Any]:
        """
        Handle rating change
        
        Args:
            value: New rating value
            
        Returns:
            Dict containing response data
        """
        old_value = self.value
        self.value = value
        is_valid = self.validate()
        
        # Calculate percentage
        percentage = (value / self.max_rating) * 100 if self.max_rating > 0 else 0
        
        # Determine star display
        full_stars = int(value)
        has_half_star = (value % 1) >= 0.5
        empty_stars = self.max_rating - full_stars - (1 if has_half_star else 0)
        
        return {
            "success": True,
            "value": self.value,
            "previousValue": old_value,
            "valid": is_valid,
            "percentage": percentage,
            "stars": {
                "full": full_stars,
                "half": 1 if has_half_star else 0,
                "empty": empty_stars
            }
        }
    
    def clear_rating(self) -> Dict[str, Any]:
        """
        Clear rating value
        
        Returns:
            Dict containing response data
        """
        return self.on_rate(0)
    
    def set_max_rating(self) -> Dict[str, Any]:
        """
        Set to maximum rating
        
        Returns:
            Dict containing response data
        """
        return self.on_rate(self.max_rating)
    
    def increment_rating(self, step: float = 1) -> Dict[str, Any]:
        """
        Increment rating by step
        
        Args:
            step: Amount to increment
            
        Returns:
            Dict containing response data
        """
        new_value = min(self.value + step, self.max_rating)
        return self.on_rate(new_value)
    
    def decrement_rating(self, step: float = 1) -> Dict[str, Any]:
        """
        Decrement rating by step
        
        Args:
            step: Amount to decrement
            
        Returns:
            Dict containing response data
        """
        new_value = max(self.value - step, 0)
        return self.on_rate(new_value)
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        props = {
            "max": self.max_rating,
            "value": self.value,
            "allowHalf": self.allow_half,
            **self.get_base_props()
        }
        
        if self.onChange:
            props["onChange"] = self.onChange
            
        return {
            "type": "rate",
            "props": props
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Rate':
        """
        Create rate component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Rate instance
        """
        return Rate(
            max_rating=data.get('max', 5),
            value=data.get('value', 0),
            allow_half=data.get('allowHalf', True),
            onChange=data.get('onChange')
        )

def create_rate(max_rating: int = 5, value: float = 0, allow_half: bool = True,
               onChange: Dict[str, Any] = None, **kwargs) -> Dict[str, Any]:
    """Create a rate component"""
    component = Rate(max_rating, value, allow_half, onChange, **kwargs)
    return component.to_dict()


"""
Theme loader module for the Cacao framework.
Handles injection of CSS and JS assets into the frontend.
"""

import os
import importlib
from typing import Dict, Any, Optional, Union

def load_theme(theme_name: str, base_path: str = None) -> Dict[str, str]:
    """
    Loads theme assets for the given theme name.
    
    Returns:
        A dictionary with 'css' and 'js' keys containing the theme's CSS and JS.
    """
    base_path = base_path or os.path.join(os.path.dirname(__file__), "themes")
    css_file = os.path.join(base_path, f"{theme_name}.css")
    js_file = os.path.join(base_path, f"{theme_name}.js")
    
    assets = {"css": "", "js": ""}
    
    try:
        with open(css_file, "r", encoding="utf-8") as f:
            assets["css"] = f.read()
    except FileNotFoundError:
        pass

    try:
        with open(js_file, "r", encoding="utf-8") as f:
            assets["js"] = f.read()
    except FileNotFoundError:
        pass

    return assets

def get_sidebar_theme(theme_options: Optional[Dict[str, Any]] = None) -> 'SidebarTheme':
    """
    Get a SidebarTheme instance with optional customizations.
    
    Args:
        theme_options: Optional dictionary with theme customization options
            Can include 'colors', 'spacing', and 'fonts' dictionaries
    
    Returns:
        A SidebarTheme instance with the specified customizations
    """
    from .sidebar_theme import SidebarTheme
    
    if theme_options is None:
        return SidebarTheme()
    
    return SidebarTheme(
        colors=theme_options.get('colors'),
        spacing=theme_options.get('spacing'),
        fonts=theme_options.get('fonts')
    )



"""
Menu Component Python Integration
"""

from typing import Dict, Any, Optional, List
from ...base import Component

class Menu(Component):
    """
    Server-side logic for menu component
    """
    
    def __init__(self, items: List[Dict[str, Any]] = None, mode: str = "horizontal", 
                 theme: str = "light", default_selected: str = None, 
                 collapsed: bool = False, **kwargs):
        """
        Initialize menu component
        
        Args:
            items: List of menu items
            mode: Menu mode (horizontal, vertical, inline)
            theme: Menu theme (light, dark)
            default_selected: Default selected item key
            collapsed: Whether menu is collapsed
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.items = items or []
        self.mode = mode
        self.theme = theme
        self.default_selected = default_selected
        self.collapsed = collapsed
        self.selected_key = default_selected
    
    def validate(self) -> bool:
        """
        Validate menu configuration
        
        Returns:
            bool: True if validation passes
        """
        # Validate mode
        valid_modes = ["horizontal", "vertical", "inline"]
        if self.mode not in valid_modes:
            return False
            
        # Validate theme
        valid_themes = ["light", "dark"]
        if self.theme not in valid_themes:
            return False
            
        return True
    
    def add_item(self, item: Dict[str, Any]) -> Dict[str, Any]:
        """
        Add item to menu
        
        Args:
            item: Menu item to add
            
        Returns:
            Dict containing response data
        """
        required_keys = ["key", "label"]
        if not all(key in item for key in required_keys):
            return {
                "success": False,
                "message": f"Item must contain keys: {required_keys}"
            }
        
        self.items.append(item)
        
        return {
            "success": True,
            "items_count": len(self.items),
            "added_item": item
        }
    
    def remove_item(self, key: str) -> Dict[str, Any]:
        """
        Remove item from menu by key
        
        Args:
            key: Key of item to remove
            
        Returns:
            Dict containing response data
        """
        original_count = len(self.items)
        self.items = [item for item in self.items if item.get("key") != key]
        
        return {
            "success": True,
            "removed": original_count != len(self.items),
            "items_count": len(self.items)
        }
    
    def select_item(self, key: str) -> Dict[str, Any]:
        """
        Select menu item by key
        
        Args:
            key: Key of item to select
            
        Returns:
            Dict containing response data
        """
        # Check if item exists
        item_exists = any(item.get("key") == key for item in self.items)
        if not item_exists:
            return {
                "success": False,
                "message": f"Item with key '{key}' not found"
            }
        
        old_selected = self.selected_key
        self.selected_key = key
        
        return {
            "success": True,
            "selected_key": self.selected_key,
            "previous_selected": old_selected
        }
    
    def set_mode(self, mode: str) -> Dict[str, Any]:
        """
        Set menu mode
        
        Args:
            mode: New menu mode
            
        Returns:
            Dict containing response data
        """
        valid_modes = ["horizontal", "vertical", "inline"]
        if mode not in valid_modes:
            return {
                "success": False,
                "message": f"Invalid mode. Valid modes: {valid_modes}"
            }
        
        old_mode = self.mode
        self.mode = mode
        
        return {
            "success": True,
            "mode": self.mode,
            "previous_mode": old_mode
        }
    
    def toggle_collapse(self) -> Dict[str, Any]:
        """
        Toggle menu collapse state
        
        Returns:
            Dict containing response data
        """
        old_collapsed = self.collapsed
        self.collapsed = not self.collapsed
        
        return {
            "success": True,
            "collapsed": self.collapsed,
            "previous_collapsed": old_collapsed
        }
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        props = {
            "items": self.items,
            "mode": self.mode,
            "theme": self.theme,
            "selectedKey": self.selected_key,
            "collapsed": self.collapsed,
            **self.get_base_props()
        }
            
        return {
            "type": "menu",
            "props": props
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Menu':
        """
        Create menu component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Menu instance
        """
        return Menu(
            items=data.get('items', []),
            mode=data.get('mode', 'horizontal'),
            theme=data.get('theme', 'light'),
            default_selected=data.get('default_selected'),
            collapsed=data.get('collapsed', False)
        )

def create_menu(items: List[Dict[str, Any]] = None, mode: str = "horizontal", 
                theme: str = "light", default_selected: str = None, 
                collapsed: bool = False, **kwargs) -> Dict[str, Any]:
    """Create a menu component"""
    component = Menu(items, mode, theme, default_selected, collapsed, **kwargs)
    return component.to_dict()


"""
Switch Component Python Integration
"""

from typing import Dict, Any, Optional
from ...base import Component

class Switch(Component):
    """
    Server-side logic for switch component
    """
    
    def __init__(self, checked: bool = False, disabled: bool = False, **kwargs):
        """
        Initialize switch component
        
        Args:
            checked: Whether switch is initially checked
            disabled: Whether switch is disabled
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.checked = checked
        self.disabled = disabled
    
    def validate(self) -> bool:
        """
        Validate switch state
        
        Returns:
            bool: True if validation passes
        """
        # Switches don't typically need validation
        return True
    
    def on_change(self, checked: bool) -> Dict[str, Any]:
        """
        Handle switch state change
        
        Args:
            checked: New checked state
            
        Returns:
            Dict containing response data
        """
        self.checked = checked
        
        # Add any business logic here
        # For example, enabling/disabling features, updating settings, etc.
        
        return {
            "success": True,
            "checked": self.checked,
            "message": f"Switch {'enabled' if checked else 'disabled'}"
        }
    
    def toggle(self) -> Dict[str, Any]:
        """
        Toggle switch state
        
        Returns:
            Dict containing response data
        """
        return self.on_change(not self.checked)
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        return {
            "type": "switch",
            "props": {
                "checked": self.checked,
                "disabled": self.disabled,
                **self.get_base_props()
            }
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Switch':
        """
        Create switch component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Switch instance
        """
        return Switch(
            checked=data.get('checked', False),
            disabled=data.get('disabled', False)
        )

def create_switch(checked: bool = False, disabled: bool = False, **kwargs) -> Dict[str, Any]:
    """Create a switch component"""
    component = Switch(checked, disabled, **kwargs)
    return component.to_dict()


"""
Button Component Python Integration
"""

from typing import Dict, Any, Optional, Callable
from ...base import Component

class Button(Component):
    """
    Server-side logic for button component
    """
    
    def __init__(self, button_type: str = "button", variant: str = "primary", 
                 size: str = "medium", disabled: bool = False, 
                 loading: bool = False, onClick: Dict[str, Any] = None, **kwargs):
        """
        Initialize button component
        
        Args:
            button_type: Type of button (button, submit, reset)
            variant: Button variant (primary, secondary, danger, success, warning)
            size: Button size (small, medium, large)
            disabled: Whether button is disabled
            loading: Whether button is in loading state
            onClick: Click handler configuration
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.button_type = button_type
        self.variant = variant
        self.size = size
        self.disabled = disabled
        self.loading = loading
        self.onClick = onClick
    
    def validate(self) -> bool:
        """
        Validate button configuration
        
        Returns:
            bool: True if validation passes
        """
        # Validate button type
        valid_types = ["button", "submit", "reset"]
        if self.button_type not in valid_types:
            return False
            
        # Validate variant
        valid_variants = ["primary", "secondary", "danger", "success", "warning", "info"]
        if self.variant not in valid_variants:
            return False
            
        # Validate size
        valid_sizes = ["small", "medium", "large"]
        if self.size not in valid_sizes:
            return False
            
        return True
    
    def set_loading(self, loading: bool) -> Dict[str, Any]:
        """
        Set loading state
        
        Args:
            loading: New loading state
            
        Returns:
            Dict containing response data
        """
        old_loading = self.loading
        self.loading = loading
        
        return {
            "success": True,
            "loading": self.loading,
            "previous_loading": old_loading
        }
    
    def set_disabled(self, disabled: bool) -> Dict[str, Any]:
        """
        Set disabled state
        
        Args:
            disabled: New disabled state
            
        Returns:
            Dict containing response data
        """
        old_disabled = self.disabled
        self.disabled = disabled
        
        return {
            "success": True,
            "disabled": self.disabled,
            "previous_disabled": old_disabled
        }
    
    def on_click(self) -> Dict[str, Any]:
        """
        Handle button click
        
        Returns:
            Dict containing response data
        """
        if self.disabled or self.loading:
            return {
                "success": False,
                "message": "Button is disabled or loading"
            }
        
        return {
            "success": True,
            "clicked": True,
            "button_type": self.button_type,
            "variant": self.variant
        }
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        props = {
            "type": self.button_type,
            "variant": self.variant,
            "size": self.size,
            "disabled": self.disabled,
            "loading": self.loading,
            **self.get_base_props()
        }
        
        if self.onClick:
            props["onClick"] = self.onClick
            
        return {
            "type": "button",
            "props": props
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Button':
        """
        Create button component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Button instance
        """
        return Button(
            button_type=data.get('type', 'button'),
            variant=data.get('variant', 'primary'),
            size=data.get('size', 'medium'),
            disabled=data.get('disabled', False),
            loading=data.get('loading', False),
            onClick=data.get('onClick')
        )

def create_button(button_type: str = "button", variant: str = "primary", 
                  size: str = "medium", disabled: bool = False, 
                  loading: bool = False, onClick: Dict[str, Any] = None, **kwargs) -> Dict[str, Any]:
    """Create a button component"""
    component = Button(button_type, variant, size, disabled, loading, onClick, **kwargs)
    return component.to_dict()


"""
Tooltip component for displaying additional information on hover.
"""

from typing import Any, Dict
from ...base import Component


class Tooltip(Component):
    """
    A small popup with additional information that appears on hover.
    """
    def __init__(
        self,
        content,
        children,
        placement = "top",
        **kwargs
    ) -> None:
        self.content = content
        self.children = children
        self.placement = placement
        self.extra_props = kwargs

    def render(self) -> Dict[str, Any]:
        return {
            "type": "tooltip",
            "props": {
                "content": self.content,
                "children": self.children.render(),
                "placement": self.placement,
                **self.extra_props
            }
        }


"""
Textarea Component Python Integration
"""

from typing import Dict, Any, Optional
from ...base import Component

class Textarea(Component):
    """
    Server-side logic for textarea component
    """
    
    def __init__(self, content: str = "", placeholder: str = None, disabled: bool = False,
                 action: str = None, **kwargs):
        """
        Initialize textarea component
        
        Args:
            content: Initial content
            placeholder: Placeholder text
            disabled: Whether textarea is disabled
            action: Action to trigger on content change
            **kwargs: Additional component properties
        """
        super().__init__(**kwargs)
        self.content = content
        self.placeholder = placeholder
        self.disabled = disabled
        self.action = action
    
    def validate(self) -> bool:
        """
        Validate textarea content
        
        Returns:
            bool: True if validation passes
        """
        # Add validation logic here (e.g., length limits, content rules)
        return True
    
    def on_change(self, content: str) -> Dict[str, Any]:
        """
        Handle textarea content change
        
        Args:
            content: New content value
            
        Returns:
            Dict containing response data
        """
        old_content = self.content
        self.content = content
        is_valid = self.validate()
        
        # Calculate statistics
        word_count = len(content.split()) if content else 0
        char_count = len(content)
        
        return {
            "success": True,
            "content": self.content,
            "previousContent": old_content,
            "valid": is_valid,
            "stats": {
                "wordCount": word_count,
                "charCount": char_count
            }
        }
    
    def clear_content(self) -> Dict[str, Any]:
        """
        Clear textarea content
        
        Returns:
            Dict containing response data
        """
        return self.on_change("")
    
    def append_content(self, text: str) -> Dict[str, Any]:
        """
        Append text to current content
        
        Args:
            text: Text to append
            
        Returns:
            Dict containing response data
        """
        new_content = self.content + text
        return self.on_change(new_content)
    
    def to_dict(self) -> Dict[str, Any]:
        """
        Convert component to dictionary for rendering
        
        Returns:
            Dict containing component data
        """
        props = {
            "content": self.content,
            "disabled": self.disabled,
            **self.get_base_props()
        }
        
        if self.placeholder:
            props["placeholder"] = self.placeholder
        if self.action:
            props["action"] = self.action
            
        return {
            "type": "textarea",
            "props": props
        }
    
    @staticmethod
    def from_form_data(data: Dict[str, Any]) -> 'Textarea':
        """
        Create textarea component from form data
        
        Args:
            data: Form data dictionary
            
        Returns:
            Textarea instance
        """
        return Textarea(
            content=data.get('content', ''),
            placeholder=data.get('placeholder'),
            disabled=data.get('disabled', False),
            action=data.get('action')
        )

def create_textarea(content: str = "", placeholder: str = None, disabled: bool = False,
                   action: str = None, **kwargs) -> Dict[str, Any]:
    """Create a textarea component"""
    component = Textarea(content, placeholder, disabled, action, **kwargs)
    return component.to_dict()


"""
Descriptions component for displaying key-value pairs of information.
"""

from typing import Any, Dict
from ...base import Component


class Descriptions(Component):
    """
    Displays key-value pairs of information.
    """
    def __init__(
        self,
        items,
        title = "",
        bordered = False,
        column = 3,
        **kwargs
    ) -> None:
        self.items = items
        self.title = title
        self.bordered = bordered
        self.column = column
        self.extra_props = kwargs

    def render(self) -> Dict[str, Any]:
        print("[DEBUG] Descriptions.render - Title:", self.title)
        print("[DEBUG] Descriptions.render - Items:", self.items)
        print("[DEBUG] Descriptions.render - Column:", self.column)
        print("[DEBUG] Descriptions.render - Bordered:", self.bordered)
        
        rendered = {
            "type": "descriptions",
            "props": {
                "items": self.items,
                "title": self.title,
                "bordered": self.bordered,
                "column": self.column,
                **self.extra_props
            }
        }
        print("[DEBUG] Descriptions.render - Final output:", rendered)
        return rendered


from abc import ABC, abstractmethod
from typing import Any, Dict, Optional
import logging

class BasePlugin(ABC):
    """
    Enhanced base class for all Cacao plugins with improved lifecycle management,
    configuration handling, and error reporting.
    """
    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.config: Dict[str, Any] = {}
        self.initialized: bool = False
        self._version: str = "1.0.0"
        self._dependencies: Dict[str, str] = {}

    @property
    def version(self) -> str:
        """Get plugin version."""
        return self._version

    @property
    def dependencies(self) -> Dict[str, str]:
        """Get plugin dependencies."""
        return self._dependencies

    def configure(self, config: Dict[str, Any]) -> None:
        """
        Configure the plugin with the provided settings.
        
        Args:
            config: Dictionary containing plugin configuration
        """
        self.validate_config(config)
        self.config = config

    def validate_config(self, config: Dict[str, Any]) -> None:
        """
        Validate the provided configuration.
        
        Args:
            config: Configuration to validate
            
        Raises:
            ValueError: If configuration is invalid
        """
        pass

    def initialize(self) -> None:
        """
        Initialize plugin resources and validate dependencies.
        
        Raises:
            RuntimeError: If initialization fails
        """
        if self.initialized:
            return
        
        try:
            self._check_dependencies()
            self._initialize_resources()
            self.initialized = True
        except Exception as e:
            self.logger.error(f"Failed to initialize plugin: {str(e)}")
            raise RuntimeError(f"Plugin initialization failed: {str(e)}")

    def cleanup(self) -> None:
        """
        Cleanup plugin resources.
        """
        if not self.initialized:
            return
        
        try:
            self._cleanup_resources()
            self.initialized = False
        except Exception as e:
            self.logger.error(f"Failed to cleanup plugin: {str(e)}")

    def _check_dependencies(self) -> None:
        """
        Check if all required dependencies are available.
        
        Raises:
            ImportError: If a required dependency is missing
        """
        for dep, version_req in self._dependencies.items():
            try:
                module = __import__(dep)
                if hasattr(module, '__version__'):
                    # Strip the >= from the version requirement
                    min_version = version_req.replace('>=', '')
                    if module.__version__ < min_version:
                        raise ImportError(
                            f"Plugin requires {dep} {version_req}, found {module.__version__}"
                        )
            except ImportError as e:
                if "No module named" in str(e):
                    raise ImportError(f"Missing required dependency: {dep} {version_req}")
                raise

    @abstractmethod
    def _initialize_resources(self) -> None:
        """
        Initialize plugin-specific resources. Must be implemented by subclasses.
        """
        pass

    @abstractmethod
    def _cleanup_resources(self) -> None:
        """
        Cleanup plugin-specific resources. Must be implemented by subclasses.
        """
        pass

    @abstractmethod
    def process(self, data: Any) -> Any:
        """
        Process the input data. Must be implemented by subclasses.
        
        Args:
            data: Input data to process
            
        Returns:
            Processed data
        """
        pass
